/* Autogenerated: src/ExtractionOCaml/word_by_word_montgomery --lang=Rust secp256k1 '2^256 - 2^32 - 977' 32 */
/* curve description: secp256k1 */
/* requested operations: (all) */
/* m = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f (from "2^256 - 2^32 - 977") */
/* machine_wordsize = 32 (from "32") */
/*                                                                    */
/* NOTE: In addition to the bounds specified above each function, all */
/*   functions synthesized for this Montgomery arithmetic require the */
/*   input to be strictly less than the prime modulus (m), and also   */
/*   require the input to be in the unique saturated representation.  */
/*   All functions also ensure that these two properties are true of  */
/*   return values.                                                   */

#![allow(unused_parens)]
#[allow(non_camel_case_types)]

pub type fiat_secp256k1_u1 = u8;
pub type fiat_secp256k1_i1 = i8;
pub type fiat_secp256k1_u2 = u8;
pub type fiat_secp256k1_i2 = i8;


/*
 * The function fiat_secp256k1_addcarryx_u32 is an addition with carry.
 * Postconditions:
 *   out1 = (arg1 + arg2 + arg3) mod 2^32
 *   out2 = ⌊(arg1 + arg2 + arg3) / 2^32⌋
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0xffffffff]
 *   arg3: [0x0 ~> 0xffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffff]
 *   out2: [0x0 ~> 0x1]
 */
#[inline]
pub fn fiat_secp256k1_addcarryx_u32(out1: &mut u32, out2: &mut fiat_secp256k1_u1, arg1: fiat_secp256k1_u1, arg2: u32, arg3: u32) -> () {
  let x1: u64 = (((arg1 as u64) + (arg2 as u64)) + (arg3 as u64));
  let x2: u32 = ((x1 & (0xffffffff as u64)) as u32);
  let x3: fiat_secp256k1_u1 = ((x1 >> 32) as fiat_secp256k1_u1);
  *out1 = x2;
  *out2 = x3;
}

/*
 * The function fiat_secp256k1_subborrowx_u32 is a subtraction with borrow.
 * Postconditions:
 *   out1 = (-arg1 + arg2 + -arg3) mod 2^32
 *   out2 = -⌊(-arg1 + arg2 + -arg3) / 2^32⌋
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0xffffffff]
 *   arg3: [0x0 ~> 0xffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffff]
 *   out2: [0x0 ~> 0x1]
 */
#[inline]
pub fn fiat_secp256k1_subborrowx_u32(out1: &mut u32, out2: &mut fiat_secp256k1_u1, arg1: fiat_secp256k1_u1, arg2: u32, arg3: u32) -> () {
  let x1: i64 = (((arg2 as i64) - (arg1 as i64)) - (arg3 as i64));
  let x2: fiat_secp256k1_i1 = ((x1 >> 32) as fiat_secp256k1_i1);
  let x3: u32 = ((x1 & (0xffffffff as i64)) as u32);
  *out1 = x3;
  *out2 = (((0x0 as fiat_secp256k1_i2) - (x2 as fiat_secp256k1_i2)) as fiat_secp256k1_u1);
}

/*
 * The function fiat_secp256k1_mulx_u32 is a multiplication, returning the full double-width result.
 * Postconditions:
 *   out1 = (arg1 * arg2) mod 2^32
 *   out2 = ⌊arg1 * arg2 / 2^32⌋
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0xffffffff]
 *   arg2: [0x0 ~> 0xffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffff]
 *   out2: [0x0 ~> 0xffffffff]
 */
#[inline]
pub fn fiat_secp256k1_mulx_u32(out1: &mut u32, out2: &mut u32, arg1: u32, arg2: u32) -> () {
  let x1: u64 = ((arg1 as u64) * (arg2 as u64));
  let x2: u32 = ((x1 & (0xffffffff as u64)) as u32);
  let x3: u32 = ((x1 >> 32) as u32);
  *out1 = x2;
  *out2 = x3;
}

/*
 * The function fiat_secp256k1_cmovznz_u32 is a single-word conditional move.
 * Postconditions:
 *   out1 = (if arg1 = 0 then arg2 else arg3)
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0xffffffff]
 *   arg3: [0x0 ~> 0xffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffff]
 */
#[inline]
pub fn fiat_secp256k1_cmovznz_u32(out1: &mut u32, arg1: fiat_secp256k1_u1, arg2: u32, arg3: u32) -> () {
  let x1: fiat_secp256k1_u1 = (!(!arg1));
  let x2: u32 = ((((((0x0 as fiat_secp256k1_i2) - (x1 as fiat_secp256k1_i2)) as fiat_secp256k1_i1) as i64) & (0xffffffff as i64)) as u32);
  let x3: u32 = ((x2 & arg3) | ((!x2) & arg2));
  *out1 = x3;
}

/*
 * The function fiat_secp256k1_mul multiplies two field elements in the Montgomery domain.
 * Preconditions:
 *   0 ≤ eval arg1 < m
 *   0 ≤ eval arg2 < m
 * Postconditions:
 *   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) * eval (from_montgomery arg2)) mod m
 *   0 ≤ eval out1 < m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 *   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
#[inline]
pub fn fiat_secp256k1_mul(out1: &mut [u32; 8], arg1: &[u32; 8], arg2: &[u32; 8]) -> () {
  let x1: u32 = (arg1[1]);
  let x2: u32 = (arg1[2]);
  let x3: u32 = (arg1[3]);
  let x4: u32 = (arg1[4]);
  let x5: u32 = (arg1[5]);
  let x6: u32 = (arg1[6]);
  let x7: u32 = (arg1[7]);
  let x8: u32 = (arg1[0]);
  let mut x9: u32 = 0;
  let mut x10: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x9, &mut x10, x8, (arg2[7]));
  let mut x11: u32 = 0;
  let mut x12: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x11, &mut x12, x8, (arg2[6]));
  let mut x13: u32 = 0;
  let mut x14: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x13, &mut x14, x8, (arg2[5]));
  let mut x15: u32 = 0;
  let mut x16: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x15, &mut x16, x8, (arg2[4]));
  let mut x17: u32 = 0;
  let mut x18: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x17, &mut x18, x8, (arg2[3]));
  let mut x19: u32 = 0;
  let mut x20: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x19, &mut x20, x8, (arg2[2]));
  let mut x21: u32 = 0;
  let mut x22: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x21, &mut x22, x8, (arg2[1]));
  let mut x23: u32 = 0;
  let mut x24: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x23, &mut x24, x8, (arg2[0]));
  let mut x25: u32 = 0;
  let mut x26: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x25, &mut x26, 0x0, x24, x21);
  let mut x27: u32 = 0;
  let mut x28: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x27, &mut x28, x26, x22, x19);
  let mut x29: u32 = 0;
  let mut x30: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x29, &mut x30, x28, x20, x17);
  let mut x31: u32 = 0;
  let mut x32: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x31, &mut x32, x30, x18, x15);
  let mut x33: u32 = 0;
  let mut x34: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x33, &mut x34, x32, x16, x13);
  let mut x35: u32 = 0;
  let mut x36: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x35, &mut x36, x34, x14, x11);
  let mut x37: u32 = 0;
  let mut x38: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x37, &mut x38, x36, x12, x9);
  let mut x39: u32 = 0;
  let mut x40: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x39, &mut x40, x38, x10, (0x0 as u32));
  let mut x41: u32 = 0;
  let mut x42: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x41, &mut x42, x23, 0xd2253531);
  let mut x43: u32 = 0;
  let mut x44: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x43, &mut x44, x41, 0xffffffff);
  let mut x45: u32 = 0;
  let mut x46: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x45, &mut x46, x41, 0xffffffff);
  let mut x47: u32 = 0;
  let mut x48: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x47, &mut x48, x41, 0xffffffff);
  let mut x49: u32 = 0;
  let mut x50: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x49, &mut x50, x41, 0xffffffff);
  let mut x51: u32 = 0;
  let mut x52: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x51, &mut x52, x41, 0xffffffff);
  let mut x53: u32 = 0;
  let mut x54: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x53, &mut x54, x41, 0xffffffff);
  let mut x55: u32 = 0;
  let mut x56: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x55, &mut x56, x41, 0xfffffffe);
  let mut x57: u32 = 0;
  let mut x58: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x57, &mut x58, x41, 0xfffffc2f);
  let mut x59: u32 = 0;
  let mut x60: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x59, &mut x60, 0x0, x58, x55);
  let mut x61: u32 = 0;
  let mut x62: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x61, &mut x62, x60, x56, x53);
  let mut x63: u32 = 0;
  let mut x64: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x63, &mut x64, x62, x54, x51);
  let mut x65: u32 = 0;
  let mut x66: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x65, &mut x66, x64, x52, x49);
  let mut x67: u32 = 0;
  let mut x68: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x67, &mut x68, x66, x50, x47);
  let mut x69: u32 = 0;
  let mut x70: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x69, &mut x70, x68, x48, x45);
  let mut x71: u32 = 0;
  let mut x72: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x71, &mut x72, x70, x46, x43);
  let mut x73: u32 = 0;
  let mut x74: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x73, &mut x74, x72, x44, (0x0 as u32));
  let mut x75: u32 = 0;
  let mut x76: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x75, &mut x76, 0x0, x23, x57);
  let mut x77: u32 = 0;
  let mut x78: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x77, &mut x78, x76, x25, x59);
  let mut x79: u32 = 0;
  let mut x80: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x79, &mut x80, x78, x27, x61);
  let mut x81: u32 = 0;
  let mut x82: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x81, &mut x82, x80, x29, x63);
  let mut x83: u32 = 0;
  let mut x84: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x83, &mut x84, x82, x31, x65);
  let mut x85: u32 = 0;
  let mut x86: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x85, &mut x86, x84, x33, x67);
  let mut x87: u32 = 0;
  let mut x88: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x87, &mut x88, x86, x35, x69);
  let mut x89: u32 = 0;
  let mut x90: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x89, &mut x90, x88, x37, x71);
  let mut x91: u32 = 0;
  let mut x92: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x91, &mut x92, x90, x39, x73);
  let mut x93: u32 = 0;
  let mut x94: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x93, &mut x94, x92, (0x0 as u32), (0x0 as u32));
  let mut x95: u32 = 0;
  let mut x96: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x95, &mut x96, x1, (arg2[7]));
  let mut x97: u32 = 0;
  let mut x98: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x97, &mut x98, x1, (arg2[6]));
  let mut x99: u32 = 0;
  let mut x100: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x99, &mut x100, x1, (arg2[5]));
  let mut x101: u32 = 0;
  let mut x102: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x101, &mut x102, x1, (arg2[4]));
  let mut x103: u32 = 0;
  let mut x104: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x103, &mut x104, x1, (arg2[3]));
  let mut x105: u32 = 0;
  let mut x106: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x105, &mut x106, x1, (arg2[2]));
  let mut x107: u32 = 0;
  let mut x108: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x107, &mut x108, x1, (arg2[1]));
  let mut x109: u32 = 0;
  let mut x110: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x109, &mut x110, x1, (arg2[0]));
  let mut x111: u32 = 0;
  let mut x112: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x111, &mut x112, 0x0, x110, x107);
  let mut x113: u32 = 0;
  let mut x114: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x113, &mut x114, x112, x108, x105);
  let mut x115: u32 = 0;
  let mut x116: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x115, &mut x116, x114, x106, x103);
  let mut x117: u32 = 0;
  let mut x118: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x117, &mut x118, x116, x104, x101);
  let mut x119: u32 = 0;
  let mut x120: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x119, &mut x120, x118, x102, x99);
  let mut x121: u32 = 0;
  let mut x122: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x121, &mut x122, x120, x100, x97);
  let mut x123: u32 = 0;
  let mut x124: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x123, &mut x124, x122, x98, x95);
  let mut x125: u32 = 0;
  let mut x126: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x125, &mut x126, x124, x96, (0x0 as u32));
  let mut x127: u32 = 0;
  let mut x128: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x127, &mut x128, 0x0, x77, x109);
  let mut x129: u32 = 0;
  let mut x130: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x129, &mut x130, x128, x79, x111);
  let mut x131: u32 = 0;
  let mut x132: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x131, &mut x132, x130, x81, x113);
  let mut x133: u32 = 0;
  let mut x134: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x133, &mut x134, x132, x83, x115);
  let mut x135: u32 = 0;
  let mut x136: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x135, &mut x136, x134, x85, x117);
  let mut x137: u32 = 0;
  let mut x138: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x137, &mut x138, x136, x87, x119);
  let mut x139: u32 = 0;
  let mut x140: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x139, &mut x140, x138, x89, x121);
  let mut x141: u32 = 0;
  let mut x142: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x141, &mut x142, x140, x91, x123);
  let mut x143: u32 = 0;
  let mut x144: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x143, &mut x144, x142, ((x93 as fiat_secp256k1_u1) as u32), x125);
  let mut x145: u32 = 0;
  let mut x146: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x145, &mut x146, x127, 0xd2253531);
  let mut x147: u32 = 0;
  let mut x148: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x147, &mut x148, x145, 0xffffffff);
  let mut x149: u32 = 0;
  let mut x150: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x149, &mut x150, x145, 0xffffffff);
  let mut x151: u32 = 0;
  let mut x152: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x151, &mut x152, x145, 0xffffffff);
  let mut x153: u32 = 0;
  let mut x154: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x153, &mut x154, x145, 0xffffffff);
  let mut x155: u32 = 0;
  let mut x156: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x155, &mut x156, x145, 0xffffffff);
  let mut x157: u32 = 0;
  let mut x158: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x157, &mut x158, x145, 0xffffffff);
  let mut x159: u32 = 0;
  let mut x160: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x159, &mut x160, x145, 0xfffffffe);
  let mut x161: u32 = 0;
  let mut x162: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x161, &mut x162, x145, 0xfffffc2f);
  let mut x163: u32 = 0;
  let mut x164: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x163, &mut x164, 0x0, x162, x159);
  let mut x165: u32 = 0;
  let mut x166: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x165, &mut x166, x164, x160, x157);
  let mut x167: u32 = 0;
  let mut x168: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x167, &mut x168, x166, x158, x155);
  let mut x169: u32 = 0;
  let mut x170: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x169, &mut x170, x168, x156, x153);
  let mut x171: u32 = 0;
  let mut x172: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x171, &mut x172, x170, x154, x151);
  let mut x173: u32 = 0;
  let mut x174: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x173, &mut x174, x172, x152, x149);
  let mut x175: u32 = 0;
  let mut x176: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x175, &mut x176, x174, x150, x147);
  let mut x177: u32 = 0;
  let mut x178: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x177, &mut x178, x176, x148, (0x0 as u32));
  let mut x179: u32 = 0;
  let mut x180: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x179, &mut x180, 0x0, x127, x161);
  let mut x181: u32 = 0;
  let mut x182: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x181, &mut x182, x180, x129, x163);
  let mut x183: u32 = 0;
  let mut x184: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x183, &mut x184, x182, x131, x165);
  let mut x185: u32 = 0;
  let mut x186: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x185, &mut x186, x184, x133, x167);
  let mut x187: u32 = 0;
  let mut x188: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x187, &mut x188, x186, x135, x169);
  let mut x189: u32 = 0;
  let mut x190: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x189, &mut x190, x188, x137, x171);
  let mut x191: u32 = 0;
  let mut x192: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x191, &mut x192, x190, x139, x173);
  let mut x193: u32 = 0;
  let mut x194: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x193, &mut x194, x192, x141, x175);
  let mut x195: u32 = 0;
  let mut x196: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x195, &mut x196, x194, x143, x177);
  let mut x197: u32 = 0;
  let mut x198: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x197, &mut x198, x196, (x144 as u32), (0x0 as u32));
  let mut x199: u32 = 0;
  let mut x200: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x199, &mut x200, x2, (arg2[7]));
  let mut x201: u32 = 0;
  let mut x202: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x201, &mut x202, x2, (arg2[6]));
  let mut x203: u32 = 0;
  let mut x204: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x203, &mut x204, x2, (arg2[5]));
  let mut x205: u32 = 0;
  let mut x206: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x205, &mut x206, x2, (arg2[4]));
  let mut x207: u32 = 0;
  let mut x208: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x207, &mut x208, x2, (arg2[3]));
  let mut x209: u32 = 0;
  let mut x210: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x209, &mut x210, x2, (arg2[2]));
  let mut x211: u32 = 0;
  let mut x212: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x211, &mut x212, x2, (arg2[1]));
  let mut x213: u32 = 0;
  let mut x214: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x213, &mut x214, x2, (arg2[0]));
  let mut x215: u32 = 0;
  let mut x216: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x215, &mut x216, 0x0, x214, x211);
  let mut x217: u32 = 0;
  let mut x218: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x217, &mut x218, x216, x212, x209);
  let mut x219: u32 = 0;
  let mut x220: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x219, &mut x220, x218, x210, x207);
  let mut x221: u32 = 0;
  let mut x222: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x221, &mut x222, x220, x208, x205);
  let mut x223: u32 = 0;
  let mut x224: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x223, &mut x224, x222, x206, x203);
  let mut x225: u32 = 0;
  let mut x226: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x225, &mut x226, x224, x204, x201);
  let mut x227: u32 = 0;
  let mut x228: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x227, &mut x228, x226, x202, x199);
  let mut x229: u32 = 0;
  let mut x230: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x229, &mut x230, x228, x200, (0x0 as u32));
  let mut x231: u32 = 0;
  let mut x232: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x231, &mut x232, 0x0, x181, x213);
  let mut x233: u32 = 0;
  let mut x234: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x233, &mut x234, x232, x183, x215);
  let mut x235: u32 = 0;
  let mut x236: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x235, &mut x236, x234, x185, x217);
  let mut x237: u32 = 0;
  let mut x238: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x237, &mut x238, x236, x187, x219);
  let mut x239: u32 = 0;
  let mut x240: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x239, &mut x240, x238, x189, x221);
  let mut x241: u32 = 0;
  let mut x242: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x241, &mut x242, x240, x191, x223);
  let mut x243: u32 = 0;
  let mut x244: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x243, &mut x244, x242, x193, x225);
  let mut x245: u32 = 0;
  let mut x246: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x245, &mut x246, x244, x195, x227);
  let mut x247: u32 = 0;
  let mut x248: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x247, &mut x248, x246, x197, x229);
  let mut x249: u32 = 0;
  let mut x250: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x249, &mut x250, x231, 0xd2253531);
  let mut x251: u32 = 0;
  let mut x252: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x251, &mut x252, x249, 0xffffffff);
  let mut x253: u32 = 0;
  let mut x254: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x253, &mut x254, x249, 0xffffffff);
  let mut x255: u32 = 0;
  let mut x256: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x255, &mut x256, x249, 0xffffffff);
  let mut x257: u32 = 0;
  let mut x258: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x257, &mut x258, x249, 0xffffffff);
  let mut x259: u32 = 0;
  let mut x260: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x259, &mut x260, x249, 0xffffffff);
  let mut x261: u32 = 0;
  let mut x262: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x261, &mut x262, x249, 0xffffffff);
  let mut x263: u32 = 0;
  let mut x264: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x263, &mut x264, x249, 0xfffffffe);
  let mut x265: u32 = 0;
  let mut x266: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x265, &mut x266, x249, 0xfffffc2f);
  let mut x267: u32 = 0;
  let mut x268: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x267, &mut x268, 0x0, x266, x263);
  let mut x269: u32 = 0;
  let mut x270: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x269, &mut x270, x268, x264, x261);
  let mut x271: u32 = 0;
  let mut x272: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x271, &mut x272, x270, x262, x259);
  let mut x273: u32 = 0;
  let mut x274: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x273, &mut x274, x272, x260, x257);
  let mut x275: u32 = 0;
  let mut x276: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x275, &mut x276, x274, x258, x255);
  let mut x277: u32 = 0;
  let mut x278: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x277, &mut x278, x276, x256, x253);
  let mut x279: u32 = 0;
  let mut x280: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x279, &mut x280, x278, x254, x251);
  let mut x281: u32 = 0;
  let mut x282: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x281, &mut x282, x280, x252, (0x0 as u32));
  let mut x283: u32 = 0;
  let mut x284: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x283, &mut x284, 0x0, x231, x265);
  let mut x285: u32 = 0;
  let mut x286: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x285, &mut x286, x284, x233, x267);
  let mut x287: u32 = 0;
  let mut x288: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x287, &mut x288, x286, x235, x269);
  let mut x289: u32 = 0;
  let mut x290: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x289, &mut x290, x288, x237, x271);
  let mut x291: u32 = 0;
  let mut x292: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x291, &mut x292, x290, x239, x273);
  let mut x293: u32 = 0;
  let mut x294: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x293, &mut x294, x292, x241, x275);
  let mut x295: u32 = 0;
  let mut x296: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x295, &mut x296, x294, x243, x277);
  let mut x297: u32 = 0;
  let mut x298: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x297, &mut x298, x296, x245, x279);
  let mut x299: u32 = 0;
  let mut x300: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x299, &mut x300, x298, x247, x281);
  let mut x301: u32 = 0;
  let mut x302: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x301, &mut x302, x300, (x248 as u32), (0x0 as u32));
  let mut x303: u32 = 0;
  let mut x304: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x303, &mut x304, x3, (arg2[7]));
  let mut x305: u32 = 0;
  let mut x306: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x305, &mut x306, x3, (arg2[6]));
  let mut x307: u32 = 0;
  let mut x308: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x307, &mut x308, x3, (arg2[5]));
  let mut x309: u32 = 0;
  let mut x310: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x309, &mut x310, x3, (arg2[4]));
  let mut x311: u32 = 0;
  let mut x312: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x311, &mut x312, x3, (arg2[3]));
  let mut x313: u32 = 0;
  let mut x314: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x313, &mut x314, x3, (arg2[2]));
  let mut x315: u32 = 0;
  let mut x316: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x315, &mut x316, x3, (arg2[1]));
  let mut x317: u32 = 0;
  let mut x318: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x317, &mut x318, x3, (arg2[0]));
  let mut x319: u32 = 0;
  let mut x320: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x319, &mut x320, 0x0, x318, x315);
  let mut x321: u32 = 0;
  let mut x322: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x321, &mut x322, x320, x316, x313);
  let mut x323: u32 = 0;
  let mut x324: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x323, &mut x324, x322, x314, x311);
  let mut x325: u32 = 0;
  let mut x326: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x325, &mut x326, x324, x312, x309);
  let mut x327: u32 = 0;
  let mut x328: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x327, &mut x328, x326, x310, x307);
  let mut x329: u32 = 0;
  let mut x330: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x329, &mut x330, x328, x308, x305);
  let mut x331: u32 = 0;
  let mut x332: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x331, &mut x332, x330, x306, x303);
  let mut x333: u32 = 0;
  let mut x334: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x333, &mut x334, x332, x304, (0x0 as u32));
  let mut x335: u32 = 0;
  let mut x336: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x335, &mut x336, 0x0, x285, x317);
  let mut x337: u32 = 0;
  let mut x338: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x337, &mut x338, x336, x287, x319);
  let mut x339: u32 = 0;
  let mut x340: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x339, &mut x340, x338, x289, x321);
  let mut x341: u32 = 0;
  let mut x342: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x341, &mut x342, x340, x291, x323);
  let mut x343: u32 = 0;
  let mut x344: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x343, &mut x344, x342, x293, x325);
  let mut x345: u32 = 0;
  let mut x346: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x345, &mut x346, x344, x295, x327);
  let mut x347: u32 = 0;
  let mut x348: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x347, &mut x348, x346, x297, x329);
  let mut x349: u32 = 0;
  let mut x350: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x349, &mut x350, x348, x299, x331);
  let mut x351: u32 = 0;
  let mut x352: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x351, &mut x352, x350, x301, x333);
  let mut x353: u32 = 0;
  let mut x354: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x353, &mut x354, x335, 0xd2253531);
  let mut x355: u32 = 0;
  let mut x356: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x355, &mut x356, x353, 0xffffffff);
  let mut x357: u32 = 0;
  let mut x358: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x357, &mut x358, x353, 0xffffffff);
  let mut x359: u32 = 0;
  let mut x360: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x359, &mut x360, x353, 0xffffffff);
  let mut x361: u32 = 0;
  let mut x362: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x361, &mut x362, x353, 0xffffffff);
  let mut x363: u32 = 0;
  let mut x364: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x363, &mut x364, x353, 0xffffffff);
  let mut x365: u32 = 0;
  let mut x366: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x365, &mut x366, x353, 0xffffffff);
  let mut x367: u32 = 0;
  let mut x368: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x367, &mut x368, x353, 0xfffffffe);
  let mut x369: u32 = 0;
  let mut x370: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x369, &mut x370, x353, 0xfffffc2f);
  let mut x371: u32 = 0;
  let mut x372: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x371, &mut x372, 0x0, x370, x367);
  let mut x373: u32 = 0;
  let mut x374: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x373, &mut x374, x372, x368, x365);
  let mut x375: u32 = 0;
  let mut x376: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x375, &mut x376, x374, x366, x363);
  let mut x377: u32 = 0;
  let mut x378: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x377, &mut x378, x376, x364, x361);
  let mut x379: u32 = 0;
  let mut x380: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x379, &mut x380, x378, x362, x359);
  let mut x381: u32 = 0;
  let mut x382: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x381, &mut x382, x380, x360, x357);
  let mut x383: u32 = 0;
  let mut x384: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x383, &mut x384, x382, x358, x355);
  let mut x385: u32 = 0;
  let mut x386: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x385, &mut x386, x384, x356, (0x0 as u32));
  let mut x387: u32 = 0;
  let mut x388: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x387, &mut x388, 0x0, x335, x369);
  let mut x389: u32 = 0;
  let mut x390: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x389, &mut x390, x388, x337, x371);
  let mut x391: u32 = 0;
  let mut x392: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x391, &mut x392, x390, x339, x373);
  let mut x393: u32 = 0;
  let mut x394: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x393, &mut x394, x392, x341, x375);
  let mut x395: u32 = 0;
  let mut x396: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x395, &mut x396, x394, x343, x377);
  let mut x397: u32 = 0;
  let mut x398: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x397, &mut x398, x396, x345, x379);
  let mut x399: u32 = 0;
  let mut x400: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x399, &mut x400, x398, x347, x381);
  let mut x401: u32 = 0;
  let mut x402: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x401, &mut x402, x400, x349, x383);
  let mut x403: u32 = 0;
  let mut x404: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x403, &mut x404, x402, x351, x385);
  let mut x405: u32 = 0;
  let mut x406: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x405, &mut x406, x404, (x352 as u32), (0x0 as u32));
  let mut x407: u32 = 0;
  let mut x408: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x407, &mut x408, x4, (arg2[7]));
  let mut x409: u32 = 0;
  let mut x410: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x409, &mut x410, x4, (arg2[6]));
  let mut x411: u32 = 0;
  let mut x412: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x411, &mut x412, x4, (arg2[5]));
  let mut x413: u32 = 0;
  let mut x414: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x413, &mut x414, x4, (arg2[4]));
  let mut x415: u32 = 0;
  let mut x416: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x415, &mut x416, x4, (arg2[3]));
  let mut x417: u32 = 0;
  let mut x418: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x417, &mut x418, x4, (arg2[2]));
  let mut x419: u32 = 0;
  let mut x420: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x419, &mut x420, x4, (arg2[1]));
  let mut x421: u32 = 0;
  let mut x422: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x421, &mut x422, x4, (arg2[0]));
  let mut x423: u32 = 0;
  let mut x424: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x423, &mut x424, 0x0, x422, x419);
  let mut x425: u32 = 0;
  let mut x426: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x425, &mut x426, x424, x420, x417);
  let mut x427: u32 = 0;
  let mut x428: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x427, &mut x428, x426, x418, x415);
  let mut x429: u32 = 0;
  let mut x430: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x429, &mut x430, x428, x416, x413);
  let mut x431: u32 = 0;
  let mut x432: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x431, &mut x432, x430, x414, x411);
  let mut x433: u32 = 0;
  let mut x434: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x433, &mut x434, x432, x412, x409);
  let mut x435: u32 = 0;
  let mut x436: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x435, &mut x436, x434, x410, x407);
  let mut x437: u32 = 0;
  let mut x438: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x437, &mut x438, x436, x408, (0x0 as u32));
  let mut x439: u32 = 0;
  let mut x440: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x439, &mut x440, 0x0, x389, x421);
  let mut x441: u32 = 0;
  let mut x442: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x441, &mut x442, x440, x391, x423);
  let mut x443: u32 = 0;
  let mut x444: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x443, &mut x444, x442, x393, x425);
  let mut x445: u32 = 0;
  let mut x446: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x445, &mut x446, x444, x395, x427);
  let mut x447: u32 = 0;
  let mut x448: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x447, &mut x448, x446, x397, x429);
  let mut x449: u32 = 0;
  let mut x450: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x449, &mut x450, x448, x399, x431);
  let mut x451: u32 = 0;
  let mut x452: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x451, &mut x452, x450, x401, x433);
  let mut x453: u32 = 0;
  let mut x454: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x453, &mut x454, x452, x403, x435);
  let mut x455: u32 = 0;
  let mut x456: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x455, &mut x456, x454, x405, x437);
  let mut x457: u32 = 0;
  let mut x458: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x457, &mut x458, x439, 0xd2253531);
  let mut x459: u32 = 0;
  let mut x460: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x459, &mut x460, x457, 0xffffffff);
  let mut x461: u32 = 0;
  let mut x462: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x461, &mut x462, x457, 0xffffffff);
  let mut x463: u32 = 0;
  let mut x464: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x463, &mut x464, x457, 0xffffffff);
  let mut x465: u32 = 0;
  let mut x466: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x465, &mut x466, x457, 0xffffffff);
  let mut x467: u32 = 0;
  let mut x468: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x467, &mut x468, x457, 0xffffffff);
  let mut x469: u32 = 0;
  let mut x470: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x469, &mut x470, x457, 0xffffffff);
  let mut x471: u32 = 0;
  let mut x472: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x471, &mut x472, x457, 0xfffffffe);
  let mut x473: u32 = 0;
  let mut x474: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x473, &mut x474, x457, 0xfffffc2f);
  let mut x475: u32 = 0;
  let mut x476: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x475, &mut x476, 0x0, x474, x471);
  let mut x477: u32 = 0;
  let mut x478: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x477, &mut x478, x476, x472, x469);
  let mut x479: u32 = 0;
  let mut x480: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x479, &mut x480, x478, x470, x467);
  let mut x481: u32 = 0;
  let mut x482: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x481, &mut x482, x480, x468, x465);
  let mut x483: u32 = 0;
  let mut x484: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x483, &mut x484, x482, x466, x463);
  let mut x485: u32 = 0;
  let mut x486: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x485, &mut x486, x484, x464, x461);
  let mut x487: u32 = 0;
  let mut x488: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x487, &mut x488, x486, x462, x459);
  let mut x489: u32 = 0;
  let mut x490: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x489, &mut x490, x488, x460, (0x0 as u32));
  let mut x491: u32 = 0;
  let mut x492: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x491, &mut x492, 0x0, x439, x473);
  let mut x493: u32 = 0;
  let mut x494: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x493, &mut x494, x492, x441, x475);
  let mut x495: u32 = 0;
  let mut x496: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x495, &mut x496, x494, x443, x477);
  let mut x497: u32 = 0;
  let mut x498: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x497, &mut x498, x496, x445, x479);
  let mut x499: u32 = 0;
  let mut x500: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x499, &mut x500, x498, x447, x481);
  let mut x501: u32 = 0;
  let mut x502: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x501, &mut x502, x500, x449, x483);
  let mut x503: u32 = 0;
  let mut x504: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x503, &mut x504, x502, x451, x485);
  let mut x505: u32 = 0;
  let mut x506: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x505, &mut x506, x504, x453, x487);
  let mut x507: u32 = 0;
  let mut x508: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x507, &mut x508, x506, x455, x489);
  let mut x509: u32 = 0;
  let mut x510: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x509, &mut x510, x508, (x456 as u32), (0x0 as u32));
  let mut x511: u32 = 0;
  let mut x512: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x511, &mut x512, x5, (arg2[7]));
  let mut x513: u32 = 0;
  let mut x514: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x513, &mut x514, x5, (arg2[6]));
  let mut x515: u32 = 0;
  let mut x516: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x515, &mut x516, x5, (arg2[5]));
  let mut x517: u32 = 0;
  let mut x518: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x517, &mut x518, x5, (arg2[4]));
  let mut x519: u32 = 0;
  let mut x520: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x519, &mut x520, x5, (arg2[3]));
  let mut x521: u32 = 0;
  let mut x522: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x521, &mut x522, x5, (arg2[2]));
  let mut x523: u32 = 0;
  let mut x524: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x523, &mut x524, x5, (arg2[1]));
  let mut x525: u32 = 0;
  let mut x526: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x525, &mut x526, x5, (arg2[0]));
  let mut x527: u32 = 0;
  let mut x528: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x527, &mut x528, 0x0, x526, x523);
  let mut x529: u32 = 0;
  let mut x530: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x529, &mut x530, x528, x524, x521);
  let mut x531: u32 = 0;
  let mut x532: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x531, &mut x532, x530, x522, x519);
  let mut x533: u32 = 0;
  let mut x534: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x533, &mut x534, x532, x520, x517);
  let mut x535: u32 = 0;
  let mut x536: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x535, &mut x536, x534, x518, x515);
  let mut x537: u32 = 0;
  let mut x538: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x537, &mut x538, x536, x516, x513);
  let mut x539: u32 = 0;
  let mut x540: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x539, &mut x540, x538, x514, x511);
  let mut x541: u32 = 0;
  let mut x542: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x541, &mut x542, x540, x512, (0x0 as u32));
  let mut x543: u32 = 0;
  let mut x544: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x543, &mut x544, 0x0, x493, x525);
  let mut x545: u32 = 0;
  let mut x546: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x545, &mut x546, x544, x495, x527);
  let mut x547: u32 = 0;
  let mut x548: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x547, &mut x548, x546, x497, x529);
  let mut x549: u32 = 0;
  let mut x550: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x549, &mut x550, x548, x499, x531);
  let mut x551: u32 = 0;
  let mut x552: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x551, &mut x552, x550, x501, x533);
  let mut x553: u32 = 0;
  let mut x554: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x553, &mut x554, x552, x503, x535);
  let mut x555: u32 = 0;
  let mut x556: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x555, &mut x556, x554, x505, x537);
  let mut x557: u32 = 0;
  let mut x558: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x557, &mut x558, x556, x507, x539);
  let mut x559: u32 = 0;
  let mut x560: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x559, &mut x560, x558, x509, x541);
  let mut x561: u32 = 0;
  let mut x562: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x561, &mut x562, x543, 0xd2253531);
  let mut x563: u32 = 0;
  let mut x564: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x563, &mut x564, x561, 0xffffffff);
  let mut x565: u32 = 0;
  let mut x566: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x565, &mut x566, x561, 0xffffffff);
  let mut x567: u32 = 0;
  let mut x568: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x567, &mut x568, x561, 0xffffffff);
  let mut x569: u32 = 0;
  let mut x570: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x569, &mut x570, x561, 0xffffffff);
  let mut x571: u32 = 0;
  let mut x572: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x571, &mut x572, x561, 0xffffffff);
  let mut x573: u32 = 0;
  let mut x574: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x573, &mut x574, x561, 0xffffffff);
  let mut x575: u32 = 0;
  let mut x576: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x575, &mut x576, x561, 0xfffffffe);
  let mut x577: u32 = 0;
  let mut x578: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x577, &mut x578, x561, 0xfffffc2f);
  let mut x579: u32 = 0;
  let mut x580: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x579, &mut x580, 0x0, x578, x575);
  let mut x581: u32 = 0;
  let mut x582: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x581, &mut x582, x580, x576, x573);
  let mut x583: u32 = 0;
  let mut x584: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x583, &mut x584, x582, x574, x571);
  let mut x585: u32 = 0;
  let mut x586: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x585, &mut x586, x584, x572, x569);
  let mut x587: u32 = 0;
  let mut x588: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x587, &mut x588, x586, x570, x567);
  let mut x589: u32 = 0;
  let mut x590: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x589, &mut x590, x588, x568, x565);
  let mut x591: u32 = 0;
  let mut x592: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x591, &mut x592, x590, x566, x563);
  let mut x593: u32 = 0;
  let mut x594: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x593, &mut x594, x592, x564, (0x0 as u32));
  let mut x595: u32 = 0;
  let mut x596: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x595, &mut x596, 0x0, x543, x577);
  let mut x597: u32 = 0;
  let mut x598: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x597, &mut x598, x596, x545, x579);
  let mut x599: u32 = 0;
  let mut x600: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x599, &mut x600, x598, x547, x581);
  let mut x601: u32 = 0;
  let mut x602: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x601, &mut x602, x600, x549, x583);
  let mut x603: u32 = 0;
  let mut x604: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x603, &mut x604, x602, x551, x585);
  let mut x605: u32 = 0;
  let mut x606: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x605, &mut x606, x604, x553, x587);
  let mut x607: u32 = 0;
  let mut x608: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x607, &mut x608, x606, x555, x589);
  let mut x609: u32 = 0;
  let mut x610: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x609, &mut x610, x608, x557, x591);
  let mut x611: u32 = 0;
  let mut x612: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x611, &mut x612, x610, x559, x593);
  let mut x613: u32 = 0;
  let mut x614: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x613, &mut x614, x612, (x560 as u32), (0x0 as u32));
  let mut x615: u32 = 0;
  let mut x616: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x615, &mut x616, x6, (arg2[7]));
  let mut x617: u32 = 0;
  let mut x618: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x617, &mut x618, x6, (arg2[6]));
  let mut x619: u32 = 0;
  let mut x620: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x619, &mut x620, x6, (arg2[5]));
  let mut x621: u32 = 0;
  let mut x622: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x621, &mut x622, x6, (arg2[4]));
  let mut x623: u32 = 0;
  let mut x624: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x623, &mut x624, x6, (arg2[3]));
  let mut x625: u32 = 0;
  let mut x626: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x625, &mut x626, x6, (arg2[2]));
  let mut x627: u32 = 0;
  let mut x628: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x627, &mut x628, x6, (arg2[1]));
  let mut x629: u32 = 0;
  let mut x630: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x629, &mut x630, x6, (arg2[0]));
  let mut x631: u32 = 0;
  let mut x632: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x631, &mut x632, 0x0, x630, x627);
  let mut x633: u32 = 0;
  let mut x634: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x633, &mut x634, x632, x628, x625);
  let mut x635: u32 = 0;
  let mut x636: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x635, &mut x636, x634, x626, x623);
  let mut x637: u32 = 0;
  let mut x638: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x637, &mut x638, x636, x624, x621);
  let mut x639: u32 = 0;
  let mut x640: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x639, &mut x640, x638, x622, x619);
  let mut x641: u32 = 0;
  let mut x642: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x641, &mut x642, x640, x620, x617);
  let mut x643: u32 = 0;
  let mut x644: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x643, &mut x644, x642, x618, x615);
  let mut x645: u32 = 0;
  let mut x646: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x645, &mut x646, x644, x616, (0x0 as u32));
  let mut x647: u32 = 0;
  let mut x648: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x647, &mut x648, 0x0, x597, x629);
  let mut x649: u32 = 0;
  let mut x650: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x649, &mut x650, x648, x599, x631);
  let mut x651: u32 = 0;
  let mut x652: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x651, &mut x652, x650, x601, x633);
  let mut x653: u32 = 0;
  let mut x654: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x653, &mut x654, x652, x603, x635);
  let mut x655: u32 = 0;
  let mut x656: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x655, &mut x656, x654, x605, x637);
  let mut x657: u32 = 0;
  let mut x658: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x657, &mut x658, x656, x607, x639);
  let mut x659: u32 = 0;
  let mut x660: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x659, &mut x660, x658, x609, x641);
  let mut x661: u32 = 0;
  let mut x662: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x661, &mut x662, x660, x611, x643);
  let mut x663: u32 = 0;
  let mut x664: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x663, &mut x664, x662, x613, x645);
  let mut x665: u32 = 0;
  let mut x666: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x665, &mut x666, x647, 0xd2253531);
  let mut x667: u32 = 0;
  let mut x668: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x667, &mut x668, x665, 0xffffffff);
  let mut x669: u32 = 0;
  let mut x670: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x669, &mut x670, x665, 0xffffffff);
  let mut x671: u32 = 0;
  let mut x672: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x671, &mut x672, x665, 0xffffffff);
  let mut x673: u32 = 0;
  let mut x674: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x673, &mut x674, x665, 0xffffffff);
  let mut x675: u32 = 0;
  let mut x676: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x675, &mut x676, x665, 0xffffffff);
  let mut x677: u32 = 0;
  let mut x678: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x677, &mut x678, x665, 0xffffffff);
  let mut x679: u32 = 0;
  let mut x680: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x679, &mut x680, x665, 0xfffffffe);
  let mut x681: u32 = 0;
  let mut x682: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x681, &mut x682, x665, 0xfffffc2f);
  let mut x683: u32 = 0;
  let mut x684: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x683, &mut x684, 0x0, x682, x679);
  let mut x685: u32 = 0;
  let mut x686: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x685, &mut x686, x684, x680, x677);
  let mut x687: u32 = 0;
  let mut x688: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x687, &mut x688, x686, x678, x675);
  let mut x689: u32 = 0;
  let mut x690: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x689, &mut x690, x688, x676, x673);
  let mut x691: u32 = 0;
  let mut x692: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x691, &mut x692, x690, x674, x671);
  let mut x693: u32 = 0;
  let mut x694: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x693, &mut x694, x692, x672, x669);
  let mut x695: u32 = 0;
  let mut x696: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x695, &mut x696, x694, x670, x667);
  let mut x697: u32 = 0;
  let mut x698: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x697, &mut x698, x696, x668, (0x0 as u32));
  let mut x699: u32 = 0;
  let mut x700: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x699, &mut x700, 0x0, x647, x681);
  let mut x701: u32 = 0;
  let mut x702: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x701, &mut x702, x700, x649, x683);
  let mut x703: u32 = 0;
  let mut x704: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x703, &mut x704, x702, x651, x685);
  let mut x705: u32 = 0;
  let mut x706: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x705, &mut x706, x704, x653, x687);
  let mut x707: u32 = 0;
  let mut x708: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x707, &mut x708, x706, x655, x689);
  let mut x709: u32 = 0;
  let mut x710: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x709, &mut x710, x708, x657, x691);
  let mut x711: u32 = 0;
  let mut x712: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x711, &mut x712, x710, x659, x693);
  let mut x713: u32 = 0;
  let mut x714: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x713, &mut x714, x712, x661, x695);
  let mut x715: u32 = 0;
  let mut x716: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x715, &mut x716, x714, x663, x697);
  let mut x717: u32 = 0;
  let mut x718: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x717, &mut x718, x716, (x664 as u32), (0x0 as u32));
  let mut x719: u32 = 0;
  let mut x720: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x719, &mut x720, x7, (arg2[7]));
  let mut x721: u32 = 0;
  let mut x722: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x721, &mut x722, x7, (arg2[6]));
  let mut x723: u32 = 0;
  let mut x724: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x723, &mut x724, x7, (arg2[5]));
  let mut x725: u32 = 0;
  let mut x726: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x725, &mut x726, x7, (arg2[4]));
  let mut x727: u32 = 0;
  let mut x728: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x727, &mut x728, x7, (arg2[3]));
  let mut x729: u32 = 0;
  let mut x730: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x729, &mut x730, x7, (arg2[2]));
  let mut x731: u32 = 0;
  let mut x732: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x731, &mut x732, x7, (arg2[1]));
  let mut x733: u32 = 0;
  let mut x734: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x733, &mut x734, x7, (arg2[0]));
  let mut x735: u32 = 0;
  let mut x736: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x735, &mut x736, 0x0, x734, x731);
  let mut x737: u32 = 0;
  let mut x738: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x737, &mut x738, x736, x732, x729);
  let mut x739: u32 = 0;
  let mut x740: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x739, &mut x740, x738, x730, x727);
  let mut x741: u32 = 0;
  let mut x742: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x741, &mut x742, x740, x728, x725);
  let mut x743: u32 = 0;
  let mut x744: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x743, &mut x744, x742, x726, x723);
  let mut x745: u32 = 0;
  let mut x746: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x745, &mut x746, x744, x724, x721);
  let mut x747: u32 = 0;
  let mut x748: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x747, &mut x748, x746, x722, x719);
  let mut x749: u32 = 0;
  let mut x750: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x749, &mut x750, x748, x720, (0x0 as u32));
  let mut x751: u32 = 0;
  let mut x752: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x751, &mut x752, 0x0, x701, x733);
  let mut x753: u32 = 0;
  let mut x754: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x753, &mut x754, x752, x703, x735);
  let mut x755: u32 = 0;
  let mut x756: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x755, &mut x756, x754, x705, x737);
  let mut x757: u32 = 0;
  let mut x758: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x757, &mut x758, x756, x707, x739);
  let mut x759: u32 = 0;
  let mut x760: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x759, &mut x760, x758, x709, x741);
  let mut x761: u32 = 0;
  let mut x762: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x761, &mut x762, x760, x711, x743);
  let mut x763: u32 = 0;
  let mut x764: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x763, &mut x764, x762, x713, x745);
  let mut x765: u32 = 0;
  let mut x766: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x765, &mut x766, x764, x715, x747);
  let mut x767: u32 = 0;
  let mut x768: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x767, &mut x768, x766, x717, x749);
  let mut x769: u32 = 0;
  let mut x770: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x769, &mut x770, x751, 0xd2253531);
  let mut x771: u32 = 0;
  let mut x772: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x771, &mut x772, x769, 0xffffffff);
  let mut x773: u32 = 0;
  let mut x774: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x773, &mut x774, x769, 0xffffffff);
  let mut x775: u32 = 0;
  let mut x776: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x775, &mut x776, x769, 0xffffffff);
  let mut x777: u32 = 0;
  let mut x778: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x777, &mut x778, x769, 0xffffffff);
  let mut x779: u32 = 0;
  let mut x780: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x779, &mut x780, x769, 0xffffffff);
  let mut x781: u32 = 0;
  let mut x782: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x781, &mut x782, x769, 0xffffffff);
  let mut x783: u32 = 0;
  let mut x784: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x783, &mut x784, x769, 0xfffffffe);
  let mut x785: u32 = 0;
  let mut x786: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x785, &mut x786, x769, 0xfffffc2f);
  let mut x787: u32 = 0;
  let mut x788: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x787, &mut x788, 0x0, x786, x783);
  let mut x789: u32 = 0;
  let mut x790: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x789, &mut x790, x788, x784, x781);
  let mut x791: u32 = 0;
  let mut x792: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x791, &mut x792, x790, x782, x779);
  let mut x793: u32 = 0;
  let mut x794: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x793, &mut x794, x792, x780, x777);
  let mut x795: u32 = 0;
  let mut x796: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x795, &mut x796, x794, x778, x775);
  let mut x797: u32 = 0;
  let mut x798: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x797, &mut x798, x796, x776, x773);
  let mut x799: u32 = 0;
  let mut x800: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x799, &mut x800, x798, x774, x771);
  let mut x801: u32 = 0;
  let mut x802: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x801, &mut x802, x800, x772, (0x0 as u32));
  let mut x803: u32 = 0;
  let mut x804: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x803, &mut x804, 0x0, x751, x785);
  let mut x805: u32 = 0;
  let mut x806: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x805, &mut x806, x804, x753, x787);
  let mut x807: u32 = 0;
  let mut x808: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x807, &mut x808, x806, x755, x789);
  let mut x809: u32 = 0;
  let mut x810: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x809, &mut x810, x808, x757, x791);
  let mut x811: u32 = 0;
  let mut x812: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x811, &mut x812, x810, x759, x793);
  let mut x813: u32 = 0;
  let mut x814: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x813, &mut x814, x812, x761, x795);
  let mut x815: u32 = 0;
  let mut x816: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x815, &mut x816, x814, x763, x797);
  let mut x817: u32 = 0;
  let mut x818: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x817, &mut x818, x816, x765, x799);
  let mut x819: u32 = 0;
  let mut x820: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x819, &mut x820, x818, x767, x801);
  let mut x821: u32 = 0;
  let mut x822: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x821, &mut x822, x820, (x768 as u32), (0x0 as u32));
  let mut x823: u32 = 0;
  let mut x824: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x823, &mut x824, 0x0, x805, 0xfffffc2f);
  let mut x825: u32 = 0;
  let mut x826: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x825, &mut x826, x824, x807, 0xfffffffe);
  let mut x827: u32 = 0;
  let mut x828: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x827, &mut x828, x826, x809, 0xffffffff);
  let mut x829: u32 = 0;
  let mut x830: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x829, &mut x830, x828, x811, 0xffffffff);
  let mut x831: u32 = 0;
  let mut x832: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x831, &mut x832, x830, x813, 0xffffffff);
  let mut x833: u32 = 0;
  let mut x834: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x833, &mut x834, x832, x815, 0xffffffff);
  let mut x835: u32 = 0;
  let mut x836: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x835, &mut x836, x834, x817, 0xffffffff);
  let mut x837: u32 = 0;
  let mut x838: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x837, &mut x838, x836, x819, 0xffffffff);
  let mut x839: u32 = 0;
  let mut x840: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x839, &mut x840, x838, x821, (0x0 as u32));
  let mut x841: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x841, x840, x823, x805);
  let mut x842: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x842, x840, x825, x807);
  let mut x843: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x843, x840, x827, x809);
  let mut x844: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x844, x840, x829, x811);
  let mut x845: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x845, x840, x831, x813);
  let mut x846: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x846, x840, x833, x815);
  let mut x847: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x847, x840, x835, x817);
  let mut x848: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x848, x840, x837, x819);
  out1[0] = x841;
  out1[1] = x842;
  out1[2] = x843;
  out1[3] = x844;
  out1[4] = x845;
  out1[5] = x846;
  out1[6] = x847;
  out1[7] = x848;
}

/*
 * The function fiat_secp256k1_square squares a field element in the Montgomery domain.
 * Preconditions:
 *   0 ≤ eval arg1 < m
 * Postconditions:
 *   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) * eval (from_montgomery arg1)) mod m
 *   0 ≤ eval out1 < m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
#[inline]
pub fn fiat_secp256k1_square(out1: &mut [u32; 8], arg1: &[u32; 8]) -> () {
  let x1: u32 = (arg1[1]);
  let x2: u32 = (arg1[2]);
  let x3: u32 = (arg1[3]);
  let x4: u32 = (arg1[4]);
  let x5: u32 = (arg1[5]);
  let x6: u32 = (arg1[6]);
  let x7: u32 = (arg1[7]);
  let x8: u32 = (arg1[0]);
  let mut x9: u32 = 0;
  let mut x10: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x9, &mut x10, x8, (arg1[7]));
  let mut x11: u32 = 0;
  let mut x12: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x11, &mut x12, x8, (arg1[6]));
  let mut x13: u32 = 0;
  let mut x14: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x13, &mut x14, x8, (arg1[5]));
  let mut x15: u32 = 0;
  let mut x16: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x15, &mut x16, x8, (arg1[4]));
  let mut x17: u32 = 0;
  let mut x18: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x17, &mut x18, x8, (arg1[3]));
  let mut x19: u32 = 0;
  let mut x20: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x19, &mut x20, x8, (arg1[2]));
  let mut x21: u32 = 0;
  let mut x22: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x21, &mut x22, x8, (arg1[1]));
  let mut x23: u32 = 0;
  let mut x24: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x23, &mut x24, x8, (arg1[0]));
  let mut x25: u32 = 0;
  let mut x26: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x25, &mut x26, 0x0, x24, x21);
  let mut x27: u32 = 0;
  let mut x28: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x27, &mut x28, x26, x22, x19);
  let mut x29: u32 = 0;
  let mut x30: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x29, &mut x30, x28, x20, x17);
  let mut x31: u32 = 0;
  let mut x32: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x31, &mut x32, x30, x18, x15);
  let mut x33: u32 = 0;
  let mut x34: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x33, &mut x34, x32, x16, x13);
  let mut x35: u32 = 0;
  let mut x36: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x35, &mut x36, x34, x14, x11);
  let mut x37: u32 = 0;
  let mut x38: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x37, &mut x38, x36, x12, x9);
  let mut x39: u32 = 0;
  let mut x40: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x39, &mut x40, x38, x10, (0x0 as u32));
  let mut x41: u32 = 0;
  let mut x42: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x41, &mut x42, x23, 0xd2253531);
  let mut x43: u32 = 0;
  let mut x44: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x43, &mut x44, x41, 0xffffffff);
  let mut x45: u32 = 0;
  let mut x46: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x45, &mut x46, x41, 0xffffffff);
  let mut x47: u32 = 0;
  let mut x48: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x47, &mut x48, x41, 0xffffffff);
  let mut x49: u32 = 0;
  let mut x50: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x49, &mut x50, x41, 0xffffffff);
  let mut x51: u32 = 0;
  let mut x52: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x51, &mut x52, x41, 0xffffffff);
  let mut x53: u32 = 0;
  let mut x54: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x53, &mut x54, x41, 0xffffffff);
  let mut x55: u32 = 0;
  let mut x56: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x55, &mut x56, x41, 0xfffffffe);
  let mut x57: u32 = 0;
  let mut x58: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x57, &mut x58, x41, 0xfffffc2f);
  let mut x59: u32 = 0;
  let mut x60: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x59, &mut x60, 0x0, x58, x55);
  let mut x61: u32 = 0;
  let mut x62: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x61, &mut x62, x60, x56, x53);
  let mut x63: u32 = 0;
  let mut x64: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x63, &mut x64, x62, x54, x51);
  let mut x65: u32 = 0;
  let mut x66: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x65, &mut x66, x64, x52, x49);
  let mut x67: u32 = 0;
  let mut x68: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x67, &mut x68, x66, x50, x47);
  let mut x69: u32 = 0;
  let mut x70: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x69, &mut x70, x68, x48, x45);
  let mut x71: u32 = 0;
  let mut x72: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x71, &mut x72, x70, x46, x43);
  let mut x73: u32 = 0;
  let mut x74: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x73, &mut x74, x72, x44, (0x0 as u32));
  let mut x75: u32 = 0;
  let mut x76: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x75, &mut x76, 0x0, x23, x57);
  let mut x77: u32 = 0;
  let mut x78: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x77, &mut x78, x76, x25, x59);
  let mut x79: u32 = 0;
  let mut x80: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x79, &mut x80, x78, x27, x61);
  let mut x81: u32 = 0;
  let mut x82: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x81, &mut x82, x80, x29, x63);
  let mut x83: u32 = 0;
  let mut x84: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x83, &mut x84, x82, x31, x65);
  let mut x85: u32 = 0;
  let mut x86: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x85, &mut x86, x84, x33, x67);
  let mut x87: u32 = 0;
  let mut x88: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x87, &mut x88, x86, x35, x69);
  let mut x89: u32 = 0;
  let mut x90: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x89, &mut x90, x88, x37, x71);
  let mut x91: u32 = 0;
  let mut x92: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x91, &mut x92, x90, x39, x73);
  let mut x93: u32 = 0;
  let mut x94: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x93, &mut x94, x92, (0x0 as u32), (0x0 as u32));
  let mut x95: u32 = 0;
  let mut x96: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x95, &mut x96, x1, (arg1[7]));
  let mut x97: u32 = 0;
  let mut x98: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x97, &mut x98, x1, (arg1[6]));
  let mut x99: u32 = 0;
  let mut x100: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x99, &mut x100, x1, (arg1[5]));
  let mut x101: u32 = 0;
  let mut x102: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x101, &mut x102, x1, (arg1[4]));
  let mut x103: u32 = 0;
  let mut x104: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x103, &mut x104, x1, (arg1[3]));
  let mut x105: u32 = 0;
  let mut x106: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x105, &mut x106, x1, (arg1[2]));
  let mut x107: u32 = 0;
  let mut x108: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x107, &mut x108, x1, (arg1[1]));
  let mut x109: u32 = 0;
  let mut x110: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x109, &mut x110, x1, (arg1[0]));
  let mut x111: u32 = 0;
  let mut x112: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x111, &mut x112, 0x0, x110, x107);
  let mut x113: u32 = 0;
  let mut x114: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x113, &mut x114, x112, x108, x105);
  let mut x115: u32 = 0;
  let mut x116: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x115, &mut x116, x114, x106, x103);
  let mut x117: u32 = 0;
  let mut x118: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x117, &mut x118, x116, x104, x101);
  let mut x119: u32 = 0;
  let mut x120: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x119, &mut x120, x118, x102, x99);
  let mut x121: u32 = 0;
  let mut x122: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x121, &mut x122, x120, x100, x97);
  let mut x123: u32 = 0;
  let mut x124: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x123, &mut x124, x122, x98, x95);
  let mut x125: u32 = 0;
  let mut x126: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x125, &mut x126, x124, x96, (0x0 as u32));
  let mut x127: u32 = 0;
  let mut x128: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x127, &mut x128, 0x0, x77, x109);
  let mut x129: u32 = 0;
  let mut x130: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x129, &mut x130, x128, x79, x111);
  let mut x131: u32 = 0;
  let mut x132: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x131, &mut x132, x130, x81, x113);
  let mut x133: u32 = 0;
  let mut x134: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x133, &mut x134, x132, x83, x115);
  let mut x135: u32 = 0;
  let mut x136: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x135, &mut x136, x134, x85, x117);
  let mut x137: u32 = 0;
  let mut x138: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x137, &mut x138, x136, x87, x119);
  let mut x139: u32 = 0;
  let mut x140: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x139, &mut x140, x138, x89, x121);
  let mut x141: u32 = 0;
  let mut x142: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x141, &mut x142, x140, x91, x123);
  let mut x143: u32 = 0;
  let mut x144: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x143, &mut x144, x142, ((x93 as fiat_secp256k1_u1) as u32), x125);
  let mut x145: u32 = 0;
  let mut x146: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x145, &mut x146, x127, 0xd2253531);
  let mut x147: u32 = 0;
  let mut x148: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x147, &mut x148, x145, 0xffffffff);
  let mut x149: u32 = 0;
  let mut x150: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x149, &mut x150, x145, 0xffffffff);
  let mut x151: u32 = 0;
  let mut x152: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x151, &mut x152, x145, 0xffffffff);
  let mut x153: u32 = 0;
  let mut x154: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x153, &mut x154, x145, 0xffffffff);
  let mut x155: u32 = 0;
  let mut x156: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x155, &mut x156, x145, 0xffffffff);
  let mut x157: u32 = 0;
  let mut x158: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x157, &mut x158, x145, 0xffffffff);
  let mut x159: u32 = 0;
  let mut x160: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x159, &mut x160, x145, 0xfffffffe);
  let mut x161: u32 = 0;
  let mut x162: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x161, &mut x162, x145, 0xfffffc2f);
  let mut x163: u32 = 0;
  let mut x164: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x163, &mut x164, 0x0, x162, x159);
  let mut x165: u32 = 0;
  let mut x166: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x165, &mut x166, x164, x160, x157);
  let mut x167: u32 = 0;
  let mut x168: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x167, &mut x168, x166, x158, x155);
  let mut x169: u32 = 0;
  let mut x170: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x169, &mut x170, x168, x156, x153);
  let mut x171: u32 = 0;
  let mut x172: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x171, &mut x172, x170, x154, x151);
  let mut x173: u32 = 0;
  let mut x174: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x173, &mut x174, x172, x152, x149);
  let mut x175: u32 = 0;
  let mut x176: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x175, &mut x176, x174, x150, x147);
  let mut x177: u32 = 0;
  let mut x178: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x177, &mut x178, x176, x148, (0x0 as u32));
  let mut x179: u32 = 0;
  let mut x180: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x179, &mut x180, 0x0, x127, x161);
  let mut x181: u32 = 0;
  let mut x182: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x181, &mut x182, x180, x129, x163);
  let mut x183: u32 = 0;
  let mut x184: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x183, &mut x184, x182, x131, x165);
  let mut x185: u32 = 0;
  let mut x186: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x185, &mut x186, x184, x133, x167);
  let mut x187: u32 = 0;
  let mut x188: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x187, &mut x188, x186, x135, x169);
  let mut x189: u32 = 0;
  let mut x190: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x189, &mut x190, x188, x137, x171);
  let mut x191: u32 = 0;
  let mut x192: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x191, &mut x192, x190, x139, x173);
  let mut x193: u32 = 0;
  let mut x194: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x193, &mut x194, x192, x141, x175);
  let mut x195: u32 = 0;
  let mut x196: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x195, &mut x196, x194, x143, x177);
  let mut x197: u32 = 0;
  let mut x198: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x197, &mut x198, x196, (x144 as u32), (0x0 as u32));
  let mut x199: u32 = 0;
  let mut x200: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x199, &mut x200, x2, (arg1[7]));
  let mut x201: u32 = 0;
  let mut x202: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x201, &mut x202, x2, (arg1[6]));
  let mut x203: u32 = 0;
  let mut x204: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x203, &mut x204, x2, (arg1[5]));
  let mut x205: u32 = 0;
  let mut x206: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x205, &mut x206, x2, (arg1[4]));
  let mut x207: u32 = 0;
  let mut x208: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x207, &mut x208, x2, (arg1[3]));
  let mut x209: u32 = 0;
  let mut x210: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x209, &mut x210, x2, (arg1[2]));
  let mut x211: u32 = 0;
  let mut x212: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x211, &mut x212, x2, (arg1[1]));
  let mut x213: u32 = 0;
  let mut x214: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x213, &mut x214, x2, (arg1[0]));
  let mut x215: u32 = 0;
  let mut x216: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x215, &mut x216, 0x0, x214, x211);
  let mut x217: u32 = 0;
  let mut x218: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x217, &mut x218, x216, x212, x209);
  let mut x219: u32 = 0;
  let mut x220: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x219, &mut x220, x218, x210, x207);
  let mut x221: u32 = 0;
  let mut x222: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x221, &mut x222, x220, x208, x205);
  let mut x223: u32 = 0;
  let mut x224: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x223, &mut x224, x222, x206, x203);
  let mut x225: u32 = 0;
  let mut x226: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x225, &mut x226, x224, x204, x201);
  let mut x227: u32 = 0;
  let mut x228: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x227, &mut x228, x226, x202, x199);
  let mut x229: u32 = 0;
  let mut x230: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x229, &mut x230, x228, x200, (0x0 as u32));
  let mut x231: u32 = 0;
  let mut x232: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x231, &mut x232, 0x0, x181, x213);
  let mut x233: u32 = 0;
  let mut x234: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x233, &mut x234, x232, x183, x215);
  let mut x235: u32 = 0;
  let mut x236: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x235, &mut x236, x234, x185, x217);
  let mut x237: u32 = 0;
  let mut x238: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x237, &mut x238, x236, x187, x219);
  let mut x239: u32 = 0;
  let mut x240: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x239, &mut x240, x238, x189, x221);
  let mut x241: u32 = 0;
  let mut x242: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x241, &mut x242, x240, x191, x223);
  let mut x243: u32 = 0;
  let mut x244: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x243, &mut x244, x242, x193, x225);
  let mut x245: u32 = 0;
  let mut x246: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x245, &mut x246, x244, x195, x227);
  let mut x247: u32 = 0;
  let mut x248: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x247, &mut x248, x246, x197, x229);
  let mut x249: u32 = 0;
  let mut x250: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x249, &mut x250, x231, 0xd2253531);
  let mut x251: u32 = 0;
  let mut x252: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x251, &mut x252, x249, 0xffffffff);
  let mut x253: u32 = 0;
  let mut x254: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x253, &mut x254, x249, 0xffffffff);
  let mut x255: u32 = 0;
  let mut x256: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x255, &mut x256, x249, 0xffffffff);
  let mut x257: u32 = 0;
  let mut x258: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x257, &mut x258, x249, 0xffffffff);
  let mut x259: u32 = 0;
  let mut x260: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x259, &mut x260, x249, 0xffffffff);
  let mut x261: u32 = 0;
  let mut x262: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x261, &mut x262, x249, 0xffffffff);
  let mut x263: u32 = 0;
  let mut x264: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x263, &mut x264, x249, 0xfffffffe);
  let mut x265: u32 = 0;
  let mut x266: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x265, &mut x266, x249, 0xfffffc2f);
  let mut x267: u32 = 0;
  let mut x268: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x267, &mut x268, 0x0, x266, x263);
  let mut x269: u32 = 0;
  let mut x270: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x269, &mut x270, x268, x264, x261);
  let mut x271: u32 = 0;
  let mut x272: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x271, &mut x272, x270, x262, x259);
  let mut x273: u32 = 0;
  let mut x274: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x273, &mut x274, x272, x260, x257);
  let mut x275: u32 = 0;
  let mut x276: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x275, &mut x276, x274, x258, x255);
  let mut x277: u32 = 0;
  let mut x278: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x277, &mut x278, x276, x256, x253);
  let mut x279: u32 = 0;
  let mut x280: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x279, &mut x280, x278, x254, x251);
  let mut x281: u32 = 0;
  let mut x282: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x281, &mut x282, x280, x252, (0x0 as u32));
  let mut x283: u32 = 0;
  let mut x284: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x283, &mut x284, 0x0, x231, x265);
  let mut x285: u32 = 0;
  let mut x286: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x285, &mut x286, x284, x233, x267);
  let mut x287: u32 = 0;
  let mut x288: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x287, &mut x288, x286, x235, x269);
  let mut x289: u32 = 0;
  let mut x290: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x289, &mut x290, x288, x237, x271);
  let mut x291: u32 = 0;
  let mut x292: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x291, &mut x292, x290, x239, x273);
  let mut x293: u32 = 0;
  let mut x294: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x293, &mut x294, x292, x241, x275);
  let mut x295: u32 = 0;
  let mut x296: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x295, &mut x296, x294, x243, x277);
  let mut x297: u32 = 0;
  let mut x298: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x297, &mut x298, x296, x245, x279);
  let mut x299: u32 = 0;
  let mut x300: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x299, &mut x300, x298, x247, x281);
  let mut x301: u32 = 0;
  let mut x302: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x301, &mut x302, x300, (x248 as u32), (0x0 as u32));
  let mut x303: u32 = 0;
  let mut x304: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x303, &mut x304, x3, (arg1[7]));
  let mut x305: u32 = 0;
  let mut x306: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x305, &mut x306, x3, (arg1[6]));
  let mut x307: u32 = 0;
  let mut x308: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x307, &mut x308, x3, (arg1[5]));
  let mut x309: u32 = 0;
  let mut x310: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x309, &mut x310, x3, (arg1[4]));
  let mut x311: u32 = 0;
  let mut x312: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x311, &mut x312, x3, (arg1[3]));
  let mut x313: u32 = 0;
  let mut x314: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x313, &mut x314, x3, (arg1[2]));
  let mut x315: u32 = 0;
  let mut x316: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x315, &mut x316, x3, (arg1[1]));
  let mut x317: u32 = 0;
  let mut x318: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x317, &mut x318, x3, (arg1[0]));
  let mut x319: u32 = 0;
  let mut x320: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x319, &mut x320, 0x0, x318, x315);
  let mut x321: u32 = 0;
  let mut x322: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x321, &mut x322, x320, x316, x313);
  let mut x323: u32 = 0;
  let mut x324: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x323, &mut x324, x322, x314, x311);
  let mut x325: u32 = 0;
  let mut x326: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x325, &mut x326, x324, x312, x309);
  let mut x327: u32 = 0;
  let mut x328: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x327, &mut x328, x326, x310, x307);
  let mut x329: u32 = 0;
  let mut x330: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x329, &mut x330, x328, x308, x305);
  let mut x331: u32 = 0;
  let mut x332: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x331, &mut x332, x330, x306, x303);
  let mut x333: u32 = 0;
  let mut x334: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x333, &mut x334, x332, x304, (0x0 as u32));
  let mut x335: u32 = 0;
  let mut x336: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x335, &mut x336, 0x0, x285, x317);
  let mut x337: u32 = 0;
  let mut x338: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x337, &mut x338, x336, x287, x319);
  let mut x339: u32 = 0;
  let mut x340: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x339, &mut x340, x338, x289, x321);
  let mut x341: u32 = 0;
  let mut x342: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x341, &mut x342, x340, x291, x323);
  let mut x343: u32 = 0;
  let mut x344: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x343, &mut x344, x342, x293, x325);
  let mut x345: u32 = 0;
  let mut x346: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x345, &mut x346, x344, x295, x327);
  let mut x347: u32 = 0;
  let mut x348: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x347, &mut x348, x346, x297, x329);
  let mut x349: u32 = 0;
  let mut x350: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x349, &mut x350, x348, x299, x331);
  let mut x351: u32 = 0;
  let mut x352: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x351, &mut x352, x350, x301, x333);
  let mut x353: u32 = 0;
  let mut x354: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x353, &mut x354, x335, 0xd2253531);
  let mut x355: u32 = 0;
  let mut x356: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x355, &mut x356, x353, 0xffffffff);
  let mut x357: u32 = 0;
  let mut x358: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x357, &mut x358, x353, 0xffffffff);
  let mut x359: u32 = 0;
  let mut x360: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x359, &mut x360, x353, 0xffffffff);
  let mut x361: u32 = 0;
  let mut x362: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x361, &mut x362, x353, 0xffffffff);
  let mut x363: u32 = 0;
  let mut x364: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x363, &mut x364, x353, 0xffffffff);
  let mut x365: u32 = 0;
  let mut x366: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x365, &mut x366, x353, 0xffffffff);
  let mut x367: u32 = 0;
  let mut x368: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x367, &mut x368, x353, 0xfffffffe);
  let mut x369: u32 = 0;
  let mut x370: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x369, &mut x370, x353, 0xfffffc2f);
  let mut x371: u32 = 0;
  let mut x372: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x371, &mut x372, 0x0, x370, x367);
  let mut x373: u32 = 0;
  let mut x374: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x373, &mut x374, x372, x368, x365);
  let mut x375: u32 = 0;
  let mut x376: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x375, &mut x376, x374, x366, x363);
  let mut x377: u32 = 0;
  let mut x378: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x377, &mut x378, x376, x364, x361);
  let mut x379: u32 = 0;
  let mut x380: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x379, &mut x380, x378, x362, x359);
  let mut x381: u32 = 0;
  let mut x382: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x381, &mut x382, x380, x360, x357);
  let mut x383: u32 = 0;
  let mut x384: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x383, &mut x384, x382, x358, x355);
  let mut x385: u32 = 0;
  let mut x386: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x385, &mut x386, x384, x356, (0x0 as u32));
  let mut x387: u32 = 0;
  let mut x388: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x387, &mut x388, 0x0, x335, x369);
  let mut x389: u32 = 0;
  let mut x390: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x389, &mut x390, x388, x337, x371);
  let mut x391: u32 = 0;
  let mut x392: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x391, &mut x392, x390, x339, x373);
  let mut x393: u32 = 0;
  let mut x394: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x393, &mut x394, x392, x341, x375);
  let mut x395: u32 = 0;
  let mut x396: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x395, &mut x396, x394, x343, x377);
  let mut x397: u32 = 0;
  let mut x398: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x397, &mut x398, x396, x345, x379);
  let mut x399: u32 = 0;
  let mut x400: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x399, &mut x400, x398, x347, x381);
  let mut x401: u32 = 0;
  let mut x402: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x401, &mut x402, x400, x349, x383);
  let mut x403: u32 = 0;
  let mut x404: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x403, &mut x404, x402, x351, x385);
  let mut x405: u32 = 0;
  let mut x406: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x405, &mut x406, x404, (x352 as u32), (0x0 as u32));
  let mut x407: u32 = 0;
  let mut x408: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x407, &mut x408, x4, (arg1[7]));
  let mut x409: u32 = 0;
  let mut x410: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x409, &mut x410, x4, (arg1[6]));
  let mut x411: u32 = 0;
  let mut x412: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x411, &mut x412, x4, (arg1[5]));
  let mut x413: u32 = 0;
  let mut x414: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x413, &mut x414, x4, (arg1[4]));
  let mut x415: u32 = 0;
  let mut x416: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x415, &mut x416, x4, (arg1[3]));
  let mut x417: u32 = 0;
  let mut x418: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x417, &mut x418, x4, (arg1[2]));
  let mut x419: u32 = 0;
  let mut x420: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x419, &mut x420, x4, (arg1[1]));
  let mut x421: u32 = 0;
  let mut x422: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x421, &mut x422, x4, (arg1[0]));
  let mut x423: u32 = 0;
  let mut x424: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x423, &mut x424, 0x0, x422, x419);
  let mut x425: u32 = 0;
  let mut x426: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x425, &mut x426, x424, x420, x417);
  let mut x427: u32 = 0;
  let mut x428: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x427, &mut x428, x426, x418, x415);
  let mut x429: u32 = 0;
  let mut x430: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x429, &mut x430, x428, x416, x413);
  let mut x431: u32 = 0;
  let mut x432: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x431, &mut x432, x430, x414, x411);
  let mut x433: u32 = 0;
  let mut x434: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x433, &mut x434, x432, x412, x409);
  let mut x435: u32 = 0;
  let mut x436: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x435, &mut x436, x434, x410, x407);
  let mut x437: u32 = 0;
  let mut x438: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x437, &mut x438, x436, x408, (0x0 as u32));
  let mut x439: u32 = 0;
  let mut x440: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x439, &mut x440, 0x0, x389, x421);
  let mut x441: u32 = 0;
  let mut x442: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x441, &mut x442, x440, x391, x423);
  let mut x443: u32 = 0;
  let mut x444: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x443, &mut x444, x442, x393, x425);
  let mut x445: u32 = 0;
  let mut x446: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x445, &mut x446, x444, x395, x427);
  let mut x447: u32 = 0;
  let mut x448: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x447, &mut x448, x446, x397, x429);
  let mut x449: u32 = 0;
  let mut x450: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x449, &mut x450, x448, x399, x431);
  let mut x451: u32 = 0;
  let mut x452: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x451, &mut x452, x450, x401, x433);
  let mut x453: u32 = 0;
  let mut x454: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x453, &mut x454, x452, x403, x435);
  let mut x455: u32 = 0;
  let mut x456: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x455, &mut x456, x454, x405, x437);
  let mut x457: u32 = 0;
  let mut x458: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x457, &mut x458, x439, 0xd2253531);
  let mut x459: u32 = 0;
  let mut x460: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x459, &mut x460, x457, 0xffffffff);
  let mut x461: u32 = 0;
  let mut x462: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x461, &mut x462, x457, 0xffffffff);
  let mut x463: u32 = 0;
  let mut x464: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x463, &mut x464, x457, 0xffffffff);
  let mut x465: u32 = 0;
  let mut x466: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x465, &mut x466, x457, 0xffffffff);
  let mut x467: u32 = 0;
  let mut x468: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x467, &mut x468, x457, 0xffffffff);
  let mut x469: u32 = 0;
  let mut x470: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x469, &mut x470, x457, 0xffffffff);
  let mut x471: u32 = 0;
  let mut x472: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x471, &mut x472, x457, 0xfffffffe);
  let mut x473: u32 = 0;
  let mut x474: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x473, &mut x474, x457, 0xfffffc2f);
  let mut x475: u32 = 0;
  let mut x476: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x475, &mut x476, 0x0, x474, x471);
  let mut x477: u32 = 0;
  let mut x478: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x477, &mut x478, x476, x472, x469);
  let mut x479: u32 = 0;
  let mut x480: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x479, &mut x480, x478, x470, x467);
  let mut x481: u32 = 0;
  let mut x482: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x481, &mut x482, x480, x468, x465);
  let mut x483: u32 = 0;
  let mut x484: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x483, &mut x484, x482, x466, x463);
  let mut x485: u32 = 0;
  let mut x486: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x485, &mut x486, x484, x464, x461);
  let mut x487: u32 = 0;
  let mut x488: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x487, &mut x488, x486, x462, x459);
  let mut x489: u32 = 0;
  let mut x490: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x489, &mut x490, x488, x460, (0x0 as u32));
  let mut x491: u32 = 0;
  let mut x492: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x491, &mut x492, 0x0, x439, x473);
  let mut x493: u32 = 0;
  let mut x494: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x493, &mut x494, x492, x441, x475);
  let mut x495: u32 = 0;
  let mut x496: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x495, &mut x496, x494, x443, x477);
  let mut x497: u32 = 0;
  let mut x498: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x497, &mut x498, x496, x445, x479);
  let mut x499: u32 = 0;
  let mut x500: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x499, &mut x500, x498, x447, x481);
  let mut x501: u32 = 0;
  let mut x502: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x501, &mut x502, x500, x449, x483);
  let mut x503: u32 = 0;
  let mut x504: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x503, &mut x504, x502, x451, x485);
  let mut x505: u32 = 0;
  let mut x506: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x505, &mut x506, x504, x453, x487);
  let mut x507: u32 = 0;
  let mut x508: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x507, &mut x508, x506, x455, x489);
  let mut x509: u32 = 0;
  let mut x510: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x509, &mut x510, x508, (x456 as u32), (0x0 as u32));
  let mut x511: u32 = 0;
  let mut x512: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x511, &mut x512, x5, (arg1[7]));
  let mut x513: u32 = 0;
  let mut x514: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x513, &mut x514, x5, (arg1[6]));
  let mut x515: u32 = 0;
  let mut x516: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x515, &mut x516, x5, (arg1[5]));
  let mut x517: u32 = 0;
  let mut x518: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x517, &mut x518, x5, (arg1[4]));
  let mut x519: u32 = 0;
  let mut x520: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x519, &mut x520, x5, (arg1[3]));
  let mut x521: u32 = 0;
  let mut x522: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x521, &mut x522, x5, (arg1[2]));
  let mut x523: u32 = 0;
  let mut x524: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x523, &mut x524, x5, (arg1[1]));
  let mut x525: u32 = 0;
  let mut x526: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x525, &mut x526, x5, (arg1[0]));
  let mut x527: u32 = 0;
  let mut x528: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x527, &mut x528, 0x0, x526, x523);
  let mut x529: u32 = 0;
  let mut x530: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x529, &mut x530, x528, x524, x521);
  let mut x531: u32 = 0;
  let mut x532: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x531, &mut x532, x530, x522, x519);
  let mut x533: u32 = 0;
  let mut x534: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x533, &mut x534, x532, x520, x517);
  let mut x535: u32 = 0;
  let mut x536: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x535, &mut x536, x534, x518, x515);
  let mut x537: u32 = 0;
  let mut x538: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x537, &mut x538, x536, x516, x513);
  let mut x539: u32 = 0;
  let mut x540: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x539, &mut x540, x538, x514, x511);
  let mut x541: u32 = 0;
  let mut x542: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x541, &mut x542, x540, x512, (0x0 as u32));
  let mut x543: u32 = 0;
  let mut x544: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x543, &mut x544, 0x0, x493, x525);
  let mut x545: u32 = 0;
  let mut x546: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x545, &mut x546, x544, x495, x527);
  let mut x547: u32 = 0;
  let mut x548: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x547, &mut x548, x546, x497, x529);
  let mut x549: u32 = 0;
  let mut x550: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x549, &mut x550, x548, x499, x531);
  let mut x551: u32 = 0;
  let mut x552: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x551, &mut x552, x550, x501, x533);
  let mut x553: u32 = 0;
  let mut x554: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x553, &mut x554, x552, x503, x535);
  let mut x555: u32 = 0;
  let mut x556: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x555, &mut x556, x554, x505, x537);
  let mut x557: u32 = 0;
  let mut x558: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x557, &mut x558, x556, x507, x539);
  let mut x559: u32 = 0;
  let mut x560: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x559, &mut x560, x558, x509, x541);
  let mut x561: u32 = 0;
  let mut x562: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x561, &mut x562, x543, 0xd2253531);
  let mut x563: u32 = 0;
  let mut x564: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x563, &mut x564, x561, 0xffffffff);
  let mut x565: u32 = 0;
  let mut x566: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x565, &mut x566, x561, 0xffffffff);
  let mut x567: u32 = 0;
  let mut x568: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x567, &mut x568, x561, 0xffffffff);
  let mut x569: u32 = 0;
  let mut x570: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x569, &mut x570, x561, 0xffffffff);
  let mut x571: u32 = 0;
  let mut x572: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x571, &mut x572, x561, 0xffffffff);
  let mut x573: u32 = 0;
  let mut x574: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x573, &mut x574, x561, 0xffffffff);
  let mut x575: u32 = 0;
  let mut x576: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x575, &mut x576, x561, 0xfffffffe);
  let mut x577: u32 = 0;
  let mut x578: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x577, &mut x578, x561, 0xfffffc2f);
  let mut x579: u32 = 0;
  let mut x580: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x579, &mut x580, 0x0, x578, x575);
  let mut x581: u32 = 0;
  let mut x582: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x581, &mut x582, x580, x576, x573);
  let mut x583: u32 = 0;
  let mut x584: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x583, &mut x584, x582, x574, x571);
  let mut x585: u32 = 0;
  let mut x586: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x585, &mut x586, x584, x572, x569);
  let mut x587: u32 = 0;
  let mut x588: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x587, &mut x588, x586, x570, x567);
  let mut x589: u32 = 0;
  let mut x590: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x589, &mut x590, x588, x568, x565);
  let mut x591: u32 = 0;
  let mut x592: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x591, &mut x592, x590, x566, x563);
  let mut x593: u32 = 0;
  let mut x594: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x593, &mut x594, x592, x564, (0x0 as u32));
  let mut x595: u32 = 0;
  let mut x596: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x595, &mut x596, 0x0, x543, x577);
  let mut x597: u32 = 0;
  let mut x598: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x597, &mut x598, x596, x545, x579);
  let mut x599: u32 = 0;
  let mut x600: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x599, &mut x600, x598, x547, x581);
  let mut x601: u32 = 0;
  let mut x602: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x601, &mut x602, x600, x549, x583);
  let mut x603: u32 = 0;
  let mut x604: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x603, &mut x604, x602, x551, x585);
  let mut x605: u32 = 0;
  let mut x606: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x605, &mut x606, x604, x553, x587);
  let mut x607: u32 = 0;
  let mut x608: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x607, &mut x608, x606, x555, x589);
  let mut x609: u32 = 0;
  let mut x610: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x609, &mut x610, x608, x557, x591);
  let mut x611: u32 = 0;
  let mut x612: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x611, &mut x612, x610, x559, x593);
  let mut x613: u32 = 0;
  let mut x614: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x613, &mut x614, x612, (x560 as u32), (0x0 as u32));
  let mut x615: u32 = 0;
  let mut x616: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x615, &mut x616, x6, (arg1[7]));
  let mut x617: u32 = 0;
  let mut x618: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x617, &mut x618, x6, (arg1[6]));
  let mut x619: u32 = 0;
  let mut x620: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x619, &mut x620, x6, (arg1[5]));
  let mut x621: u32 = 0;
  let mut x622: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x621, &mut x622, x6, (arg1[4]));
  let mut x623: u32 = 0;
  let mut x624: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x623, &mut x624, x6, (arg1[3]));
  let mut x625: u32 = 0;
  let mut x626: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x625, &mut x626, x6, (arg1[2]));
  let mut x627: u32 = 0;
  let mut x628: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x627, &mut x628, x6, (arg1[1]));
  let mut x629: u32 = 0;
  let mut x630: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x629, &mut x630, x6, (arg1[0]));
  let mut x631: u32 = 0;
  let mut x632: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x631, &mut x632, 0x0, x630, x627);
  let mut x633: u32 = 0;
  let mut x634: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x633, &mut x634, x632, x628, x625);
  let mut x635: u32 = 0;
  let mut x636: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x635, &mut x636, x634, x626, x623);
  let mut x637: u32 = 0;
  let mut x638: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x637, &mut x638, x636, x624, x621);
  let mut x639: u32 = 0;
  let mut x640: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x639, &mut x640, x638, x622, x619);
  let mut x641: u32 = 0;
  let mut x642: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x641, &mut x642, x640, x620, x617);
  let mut x643: u32 = 0;
  let mut x644: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x643, &mut x644, x642, x618, x615);
  let mut x645: u32 = 0;
  let mut x646: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x645, &mut x646, x644, x616, (0x0 as u32));
  let mut x647: u32 = 0;
  let mut x648: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x647, &mut x648, 0x0, x597, x629);
  let mut x649: u32 = 0;
  let mut x650: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x649, &mut x650, x648, x599, x631);
  let mut x651: u32 = 0;
  let mut x652: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x651, &mut x652, x650, x601, x633);
  let mut x653: u32 = 0;
  let mut x654: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x653, &mut x654, x652, x603, x635);
  let mut x655: u32 = 0;
  let mut x656: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x655, &mut x656, x654, x605, x637);
  let mut x657: u32 = 0;
  let mut x658: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x657, &mut x658, x656, x607, x639);
  let mut x659: u32 = 0;
  let mut x660: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x659, &mut x660, x658, x609, x641);
  let mut x661: u32 = 0;
  let mut x662: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x661, &mut x662, x660, x611, x643);
  let mut x663: u32 = 0;
  let mut x664: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x663, &mut x664, x662, x613, x645);
  let mut x665: u32 = 0;
  let mut x666: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x665, &mut x666, x647, 0xd2253531);
  let mut x667: u32 = 0;
  let mut x668: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x667, &mut x668, x665, 0xffffffff);
  let mut x669: u32 = 0;
  let mut x670: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x669, &mut x670, x665, 0xffffffff);
  let mut x671: u32 = 0;
  let mut x672: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x671, &mut x672, x665, 0xffffffff);
  let mut x673: u32 = 0;
  let mut x674: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x673, &mut x674, x665, 0xffffffff);
  let mut x675: u32 = 0;
  let mut x676: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x675, &mut x676, x665, 0xffffffff);
  let mut x677: u32 = 0;
  let mut x678: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x677, &mut x678, x665, 0xffffffff);
  let mut x679: u32 = 0;
  let mut x680: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x679, &mut x680, x665, 0xfffffffe);
  let mut x681: u32 = 0;
  let mut x682: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x681, &mut x682, x665, 0xfffffc2f);
  let mut x683: u32 = 0;
  let mut x684: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x683, &mut x684, 0x0, x682, x679);
  let mut x685: u32 = 0;
  let mut x686: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x685, &mut x686, x684, x680, x677);
  let mut x687: u32 = 0;
  let mut x688: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x687, &mut x688, x686, x678, x675);
  let mut x689: u32 = 0;
  let mut x690: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x689, &mut x690, x688, x676, x673);
  let mut x691: u32 = 0;
  let mut x692: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x691, &mut x692, x690, x674, x671);
  let mut x693: u32 = 0;
  let mut x694: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x693, &mut x694, x692, x672, x669);
  let mut x695: u32 = 0;
  let mut x696: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x695, &mut x696, x694, x670, x667);
  let mut x697: u32 = 0;
  let mut x698: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x697, &mut x698, x696, x668, (0x0 as u32));
  let mut x699: u32 = 0;
  let mut x700: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x699, &mut x700, 0x0, x647, x681);
  let mut x701: u32 = 0;
  let mut x702: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x701, &mut x702, x700, x649, x683);
  let mut x703: u32 = 0;
  let mut x704: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x703, &mut x704, x702, x651, x685);
  let mut x705: u32 = 0;
  let mut x706: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x705, &mut x706, x704, x653, x687);
  let mut x707: u32 = 0;
  let mut x708: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x707, &mut x708, x706, x655, x689);
  let mut x709: u32 = 0;
  let mut x710: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x709, &mut x710, x708, x657, x691);
  let mut x711: u32 = 0;
  let mut x712: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x711, &mut x712, x710, x659, x693);
  let mut x713: u32 = 0;
  let mut x714: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x713, &mut x714, x712, x661, x695);
  let mut x715: u32 = 0;
  let mut x716: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x715, &mut x716, x714, x663, x697);
  let mut x717: u32 = 0;
  let mut x718: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x717, &mut x718, x716, (x664 as u32), (0x0 as u32));
  let mut x719: u32 = 0;
  let mut x720: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x719, &mut x720, x7, (arg1[7]));
  let mut x721: u32 = 0;
  let mut x722: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x721, &mut x722, x7, (arg1[6]));
  let mut x723: u32 = 0;
  let mut x724: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x723, &mut x724, x7, (arg1[5]));
  let mut x725: u32 = 0;
  let mut x726: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x725, &mut x726, x7, (arg1[4]));
  let mut x727: u32 = 0;
  let mut x728: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x727, &mut x728, x7, (arg1[3]));
  let mut x729: u32 = 0;
  let mut x730: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x729, &mut x730, x7, (arg1[2]));
  let mut x731: u32 = 0;
  let mut x732: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x731, &mut x732, x7, (arg1[1]));
  let mut x733: u32 = 0;
  let mut x734: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x733, &mut x734, x7, (arg1[0]));
  let mut x735: u32 = 0;
  let mut x736: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x735, &mut x736, 0x0, x734, x731);
  let mut x737: u32 = 0;
  let mut x738: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x737, &mut x738, x736, x732, x729);
  let mut x739: u32 = 0;
  let mut x740: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x739, &mut x740, x738, x730, x727);
  let mut x741: u32 = 0;
  let mut x742: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x741, &mut x742, x740, x728, x725);
  let mut x743: u32 = 0;
  let mut x744: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x743, &mut x744, x742, x726, x723);
  let mut x745: u32 = 0;
  let mut x746: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x745, &mut x746, x744, x724, x721);
  let mut x747: u32 = 0;
  let mut x748: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x747, &mut x748, x746, x722, x719);
  let mut x749: u32 = 0;
  let mut x750: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x749, &mut x750, x748, x720, (0x0 as u32));
  let mut x751: u32 = 0;
  let mut x752: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x751, &mut x752, 0x0, x701, x733);
  let mut x753: u32 = 0;
  let mut x754: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x753, &mut x754, x752, x703, x735);
  let mut x755: u32 = 0;
  let mut x756: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x755, &mut x756, x754, x705, x737);
  let mut x757: u32 = 0;
  let mut x758: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x757, &mut x758, x756, x707, x739);
  let mut x759: u32 = 0;
  let mut x760: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x759, &mut x760, x758, x709, x741);
  let mut x761: u32 = 0;
  let mut x762: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x761, &mut x762, x760, x711, x743);
  let mut x763: u32 = 0;
  let mut x764: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x763, &mut x764, x762, x713, x745);
  let mut x765: u32 = 0;
  let mut x766: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x765, &mut x766, x764, x715, x747);
  let mut x767: u32 = 0;
  let mut x768: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x767, &mut x768, x766, x717, x749);
  let mut x769: u32 = 0;
  let mut x770: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x769, &mut x770, x751, 0xd2253531);
  let mut x771: u32 = 0;
  let mut x772: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x771, &mut x772, x769, 0xffffffff);
  let mut x773: u32 = 0;
  let mut x774: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x773, &mut x774, x769, 0xffffffff);
  let mut x775: u32 = 0;
  let mut x776: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x775, &mut x776, x769, 0xffffffff);
  let mut x777: u32 = 0;
  let mut x778: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x777, &mut x778, x769, 0xffffffff);
  let mut x779: u32 = 0;
  let mut x780: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x779, &mut x780, x769, 0xffffffff);
  let mut x781: u32 = 0;
  let mut x782: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x781, &mut x782, x769, 0xffffffff);
  let mut x783: u32 = 0;
  let mut x784: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x783, &mut x784, x769, 0xfffffffe);
  let mut x785: u32 = 0;
  let mut x786: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x785, &mut x786, x769, 0xfffffc2f);
  let mut x787: u32 = 0;
  let mut x788: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x787, &mut x788, 0x0, x786, x783);
  let mut x789: u32 = 0;
  let mut x790: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x789, &mut x790, x788, x784, x781);
  let mut x791: u32 = 0;
  let mut x792: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x791, &mut x792, x790, x782, x779);
  let mut x793: u32 = 0;
  let mut x794: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x793, &mut x794, x792, x780, x777);
  let mut x795: u32 = 0;
  let mut x796: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x795, &mut x796, x794, x778, x775);
  let mut x797: u32 = 0;
  let mut x798: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x797, &mut x798, x796, x776, x773);
  let mut x799: u32 = 0;
  let mut x800: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x799, &mut x800, x798, x774, x771);
  let mut x801: u32 = 0;
  let mut x802: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x801, &mut x802, x800, x772, (0x0 as u32));
  let mut x803: u32 = 0;
  let mut x804: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x803, &mut x804, 0x0, x751, x785);
  let mut x805: u32 = 0;
  let mut x806: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x805, &mut x806, x804, x753, x787);
  let mut x807: u32 = 0;
  let mut x808: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x807, &mut x808, x806, x755, x789);
  let mut x809: u32 = 0;
  let mut x810: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x809, &mut x810, x808, x757, x791);
  let mut x811: u32 = 0;
  let mut x812: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x811, &mut x812, x810, x759, x793);
  let mut x813: u32 = 0;
  let mut x814: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x813, &mut x814, x812, x761, x795);
  let mut x815: u32 = 0;
  let mut x816: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x815, &mut x816, x814, x763, x797);
  let mut x817: u32 = 0;
  let mut x818: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x817, &mut x818, x816, x765, x799);
  let mut x819: u32 = 0;
  let mut x820: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x819, &mut x820, x818, x767, x801);
  let mut x821: u32 = 0;
  let mut x822: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x821, &mut x822, x820, (x768 as u32), (0x0 as u32));
  let mut x823: u32 = 0;
  let mut x824: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x823, &mut x824, 0x0, x805, 0xfffffc2f);
  let mut x825: u32 = 0;
  let mut x826: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x825, &mut x826, x824, x807, 0xfffffffe);
  let mut x827: u32 = 0;
  let mut x828: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x827, &mut x828, x826, x809, 0xffffffff);
  let mut x829: u32 = 0;
  let mut x830: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x829, &mut x830, x828, x811, 0xffffffff);
  let mut x831: u32 = 0;
  let mut x832: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x831, &mut x832, x830, x813, 0xffffffff);
  let mut x833: u32 = 0;
  let mut x834: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x833, &mut x834, x832, x815, 0xffffffff);
  let mut x835: u32 = 0;
  let mut x836: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x835, &mut x836, x834, x817, 0xffffffff);
  let mut x837: u32 = 0;
  let mut x838: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x837, &mut x838, x836, x819, 0xffffffff);
  let mut x839: u32 = 0;
  let mut x840: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x839, &mut x840, x838, x821, (0x0 as u32));
  let mut x841: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x841, x840, x823, x805);
  let mut x842: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x842, x840, x825, x807);
  let mut x843: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x843, x840, x827, x809);
  let mut x844: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x844, x840, x829, x811);
  let mut x845: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x845, x840, x831, x813);
  let mut x846: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x846, x840, x833, x815);
  let mut x847: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x847, x840, x835, x817);
  let mut x848: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x848, x840, x837, x819);
  out1[0] = x841;
  out1[1] = x842;
  out1[2] = x843;
  out1[3] = x844;
  out1[4] = x845;
  out1[5] = x846;
  out1[6] = x847;
  out1[7] = x848;
}

/*
 * The function fiat_secp256k1_add adds two field elements in the Montgomery domain.
 * Preconditions:
 *   0 ≤ eval arg1 < m
 *   0 ≤ eval arg2 < m
 * Postconditions:
 *   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) + eval (from_montgomery arg2)) mod m
 *   0 ≤ eval out1 < m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 *   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
#[inline]
pub fn fiat_secp256k1_add(out1: &mut [u32; 8], arg1: &[u32; 8], arg2: &[u32; 8]) -> () {
  let mut x1: u32 = 0;
  let mut x2: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x1, &mut x2, 0x0, (arg1[0]), (arg2[0]));
  let mut x3: u32 = 0;
  let mut x4: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x3, &mut x4, x2, (arg1[1]), (arg2[1]));
  let mut x5: u32 = 0;
  let mut x6: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x5, &mut x6, x4, (arg1[2]), (arg2[2]));
  let mut x7: u32 = 0;
  let mut x8: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x7, &mut x8, x6, (arg1[3]), (arg2[3]));
  let mut x9: u32 = 0;
  let mut x10: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x9, &mut x10, x8, (arg1[4]), (arg2[4]));
  let mut x11: u32 = 0;
  let mut x12: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x11, &mut x12, x10, (arg1[5]), (arg2[5]));
  let mut x13: u32 = 0;
  let mut x14: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x13, &mut x14, x12, (arg1[6]), (arg2[6]));
  let mut x15: u32 = 0;
  let mut x16: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x15, &mut x16, x14, (arg1[7]), (arg2[7]));
  let mut x17: u32 = 0;
  let mut x18: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x17, &mut x18, 0x0, x1, 0xfffffc2f);
  let mut x19: u32 = 0;
  let mut x20: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x19, &mut x20, x18, x3, 0xfffffffe);
  let mut x21: u32 = 0;
  let mut x22: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x21, &mut x22, x20, x5, 0xffffffff);
  let mut x23: u32 = 0;
  let mut x24: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x23, &mut x24, x22, x7, 0xffffffff);
  let mut x25: u32 = 0;
  let mut x26: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x25, &mut x26, x24, x9, 0xffffffff);
  let mut x27: u32 = 0;
  let mut x28: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x27, &mut x28, x26, x11, 0xffffffff);
  let mut x29: u32 = 0;
  let mut x30: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x29, &mut x30, x28, x13, 0xffffffff);
  let mut x31: u32 = 0;
  let mut x32: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x31, &mut x32, x30, x15, 0xffffffff);
  let mut x33: u32 = 0;
  let mut x34: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x33, &mut x34, x32, (x16 as u32), (0x0 as u32));
  let mut x35: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x35, x34, x17, x1);
  let mut x36: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x36, x34, x19, x3);
  let mut x37: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x37, x34, x21, x5);
  let mut x38: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x38, x34, x23, x7);
  let mut x39: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x39, x34, x25, x9);
  let mut x40: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x40, x34, x27, x11);
  let mut x41: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x41, x34, x29, x13);
  let mut x42: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x42, x34, x31, x15);
  out1[0] = x35;
  out1[1] = x36;
  out1[2] = x37;
  out1[3] = x38;
  out1[4] = x39;
  out1[5] = x40;
  out1[6] = x41;
  out1[7] = x42;
}

/*
 * The function fiat_secp256k1_sub subtracts two field elements in the Montgomery domain.
 * Preconditions:
 *   0 ≤ eval arg1 < m
 *   0 ≤ eval arg2 < m
 * Postconditions:
 *   eval (from_montgomery out1) mod m = (eval (from_montgomery arg1) - eval (from_montgomery arg2)) mod m
 *   0 ≤ eval out1 < m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 *   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
#[inline]
pub fn fiat_secp256k1_sub(out1: &mut [u32; 8], arg1: &[u32; 8], arg2: &[u32; 8]) -> () {
  let mut x1: u32 = 0;
  let mut x2: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x1, &mut x2, 0x0, (arg1[0]), (arg2[0]));
  let mut x3: u32 = 0;
  let mut x4: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x3, &mut x4, x2, (arg1[1]), (arg2[1]));
  let mut x5: u32 = 0;
  let mut x6: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x5, &mut x6, x4, (arg1[2]), (arg2[2]));
  let mut x7: u32 = 0;
  let mut x8: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x7, &mut x8, x6, (arg1[3]), (arg2[3]));
  let mut x9: u32 = 0;
  let mut x10: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x9, &mut x10, x8, (arg1[4]), (arg2[4]));
  let mut x11: u32 = 0;
  let mut x12: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x11, &mut x12, x10, (arg1[5]), (arg2[5]));
  let mut x13: u32 = 0;
  let mut x14: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x13, &mut x14, x12, (arg1[6]), (arg2[6]));
  let mut x15: u32 = 0;
  let mut x16: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x15, &mut x16, x14, (arg1[7]), (arg2[7]));
  let mut x17: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x17, x16, (0x0 as u32), 0xffffffff);
  let mut x18: u32 = 0;
  let mut x19: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x18, &mut x19, 0x0, x1, (x17 & 0xfffffc2f));
  let mut x20: u32 = 0;
  let mut x21: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x20, &mut x21, x19, x3, (x17 & 0xfffffffe));
  let mut x22: u32 = 0;
  let mut x23: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x22, &mut x23, x21, x5, (x17 & 0xffffffff));
  let mut x24: u32 = 0;
  let mut x25: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x24, &mut x25, x23, x7, (x17 & 0xffffffff));
  let mut x26: u32 = 0;
  let mut x27: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x26, &mut x27, x25, x9, (x17 & 0xffffffff));
  let mut x28: u32 = 0;
  let mut x29: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x28, &mut x29, x27, x11, (x17 & 0xffffffff));
  let mut x30: u32 = 0;
  let mut x31: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x30, &mut x31, x29, x13, (x17 & 0xffffffff));
  let mut x32: u32 = 0;
  let mut x33: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x32, &mut x33, x31, x15, (x17 & 0xffffffff));
  out1[0] = x18;
  out1[1] = x20;
  out1[2] = x22;
  out1[3] = x24;
  out1[4] = x26;
  out1[5] = x28;
  out1[6] = x30;
  out1[7] = x32;
}

/*
 * The function fiat_secp256k1_opp negates a field element in the Montgomery domain.
 * Preconditions:
 *   0 ≤ eval arg1 < m
 * Postconditions:
 *   eval (from_montgomery out1) mod m = -eval (from_montgomery arg1) mod m
 *   0 ≤ eval out1 < m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
#[inline]
pub fn fiat_secp256k1_opp(out1: &mut [u32; 8], arg1: &[u32; 8]) -> () {
  let mut x1: u32 = 0;
  let mut x2: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x1, &mut x2, 0x0, (0x0 as u32), (arg1[0]));
  let mut x3: u32 = 0;
  let mut x4: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x3, &mut x4, x2, (0x0 as u32), (arg1[1]));
  let mut x5: u32 = 0;
  let mut x6: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x5, &mut x6, x4, (0x0 as u32), (arg1[2]));
  let mut x7: u32 = 0;
  let mut x8: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x7, &mut x8, x6, (0x0 as u32), (arg1[3]));
  let mut x9: u32 = 0;
  let mut x10: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x9, &mut x10, x8, (0x0 as u32), (arg1[4]));
  let mut x11: u32 = 0;
  let mut x12: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x11, &mut x12, x10, (0x0 as u32), (arg1[5]));
  let mut x13: u32 = 0;
  let mut x14: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x13, &mut x14, x12, (0x0 as u32), (arg1[6]));
  let mut x15: u32 = 0;
  let mut x16: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x15, &mut x16, x14, (0x0 as u32), (arg1[7]));
  let mut x17: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x17, x16, (0x0 as u32), 0xffffffff);
  let mut x18: u32 = 0;
  let mut x19: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x18, &mut x19, 0x0, x1, (x17 & 0xfffffc2f));
  let mut x20: u32 = 0;
  let mut x21: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x20, &mut x21, x19, x3, (x17 & 0xfffffffe));
  let mut x22: u32 = 0;
  let mut x23: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x22, &mut x23, x21, x5, (x17 & 0xffffffff));
  let mut x24: u32 = 0;
  let mut x25: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x24, &mut x25, x23, x7, (x17 & 0xffffffff));
  let mut x26: u32 = 0;
  let mut x27: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x26, &mut x27, x25, x9, (x17 & 0xffffffff));
  let mut x28: u32 = 0;
  let mut x29: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x28, &mut x29, x27, x11, (x17 & 0xffffffff));
  let mut x30: u32 = 0;
  let mut x31: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x30, &mut x31, x29, x13, (x17 & 0xffffffff));
  let mut x32: u32 = 0;
  let mut x33: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x32, &mut x33, x31, x15, (x17 & 0xffffffff));
  out1[0] = x18;
  out1[1] = x20;
  out1[2] = x22;
  out1[3] = x24;
  out1[4] = x26;
  out1[5] = x28;
  out1[6] = x30;
  out1[7] = x32;
}

/*
 * The function fiat_secp256k1_from_montgomery translates a field element out of the Montgomery domain.
 * Preconditions:
 *   0 ≤ eval arg1 < m
 * Postconditions:
 *   eval out1 mod m = (eval arg1 * ((2^32)⁻¹ mod m)^8) mod m
 *   0 ≤ eval out1 < m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
#[inline]
pub fn fiat_secp256k1_from_montgomery(out1: &mut [u32; 8], arg1: &[u32; 8]) -> () {
  let x1: u32 = (arg1[0]);
  let mut x2: u32 = 0;
  let mut x3: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x2, &mut x3, x1, 0xd2253531);
  let mut x4: u32 = 0;
  let mut x5: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x4, &mut x5, x2, 0xffffffff);
  let mut x6: u32 = 0;
  let mut x7: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x6, &mut x7, x2, 0xffffffff);
  let mut x8: u32 = 0;
  let mut x9: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x8, &mut x9, x2, 0xffffffff);
  let mut x10: u32 = 0;
  let mut x11: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x10, &mut x11, x2, 0xffffffff);
  let mut x12: u32 = 0;
  let mut x13: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x12, &mut x13, x2, 0xffffffff);
  let mut x14: u32 = 0;
  let mut x15: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x14, &mut x15, x2, 0xffffffff);
  let mut x16: u32 = 0;
  let mut x17: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x16, &mut x17, x2, 0xfffffffe);
  let mut x18: u32 = 0;
  let mut x19: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x18, &mut x19, x2, 0xfffffc2f);
  let mut x20: u32 = 0;
  let mut x21: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x20, &mut x21, 0x0, x19, x16);
  let mut x22: u32 = 0;
  let mut x23: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x22, &mut x23, x21, x17, x14);
  let mut x24: u32 = 0;
  let mut x25: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x24, &mut x25, x23, x15, x12);
  let mut x26: u32 = 0;
  let mut x27: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x26, &mut x27, x25, x13, x10);
  let mut x28: u32 = 0;
  let mut x29: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x28, &mut x29, x27, x11, x8);
  let mut x30: u32 = 0;
  let mut x31: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x30, &mut x31, x29, x9, x6);
  let mut x32: u32 = 0;
  let mut x33: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x32, &mut x33, x31, x7, x4);
  let mut x34: u32 = 0;
  let mut x35: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x34, &mut x35, 0x0, x1, x18);
  let mut x36: u32 = 0;
  let mut x37: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x36, &mut x37, x35, (0x0 as u32), x20);
  let mut x38: u32 = 0;
  let mut x39: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x38, &mut x39, x37, (0x0 as u32), x22);
  let mut x40: u32 = 0;
  let mut x41: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x40, &mut x41, x39, (0x0 as u32), x24);
  let mut x42: u32 = 0;
  let mut x43: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x42, &mut x43, x41, (0x0 as u32), x26);
  let mut x44: u32 = 0;
  let mut x45: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x44, &mut x45, x43, (0x0 as u32), x28);
  let mut x46: u32 = 0;
  let mut x47: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x46, &mut x47, x45, (0x0 as u32), x30);
  let mut x48: u32 = 0;
  let mut x49: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x48, &mut x49, x47, (0x0 as u32), x32);
  let mut x50: u32 = 0;
  let mut x51: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x50, &mut x51, x33, x5, (0x0 as u32));
  let mut x52: u32 = 0;
  let mut x53: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x52, &mut x53, x49, (0x0 as u32), x50);
  let mut x54: u32 = 0;
  let mut x55: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x54, &mut x55, 0x0, x36, (arg1[1]));
  let mut x56: u32 = 0;
  let mut x57: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x56, &mut x57, x55, x38, (0x0 as u32));
  let mut x58: u32 = 0;
  let mut x59: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x58, &mut x59, x57, x40, (0x0 as u32));
  let mut x60: u32 = 0;
  let mut x61: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x60, &mut x61, x59, x42, (0x0 as u32));
  let mut x62: u32 = 0;
  let mut x63: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x62, &mut x63, x61, x44, (0x0 as u32));
  let mut x64: u32 = 0;
  let mut x65: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x64, &mut x65, x63, x46, (0x0 as u32));
  let mut x66: u32 = 0;
  let mut x67: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x66, &mut x67, x65, x48, (0x0 as u32));
  let mut x68: u32 = 0;
  let mut x69: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x68, &mut x69, x67, x52, (0x0 as u32));
  let mut x70: u32 = 0;
  let mut x71: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x70, &mut x71, x54, 0xd2253531);
  let mut x72: u32 = 0;
  let mut x73: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x72, &mut x73, x70, 0xffffffff);
  let mut x74: u32 = 0;
  let mut x75: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x74, &mut x75, x70, 0xffffffff);
  let mut x76: u32 = 0;
  let mut x77: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x76, &mut x77, x70, 0xffffffff);
  let mut x78: u32 = 0;
  let mut x79: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x78, &mut x79, x70, 0xffffffff);
  let mut x80: u32 = 0;
  let mut x81: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x80, &mut x81, x70, 0xffffffff);
  let mut x82: u32 = 0;
  let mut x83: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x82, &mut x83, x70, 0xffffffff);
  let mut x84: u32 = 0;
  let mut x85: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x84, &mut x85, x70, 0xfffffffe);
  let mut x86: u32 = 0;
  let mut x87: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x86, &mut x87, x70, 0xfffffc2f);
  let mut x88: u32 = 0;
  let mut x89: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x88, &mut x89, 0x0, x87, x84);
  let mut x90: u32 = 0;
  let mut x91: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x90, &mut x91, x89, x85, x82);
  let mut x92: u32 = 0;
  let mut x93: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x92, &mut x93, x91, x83, x80);
  let mut x94: u32 = 0;
  let mut x95: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x94, &mut x95, x93, x81, x78);
  let mut x96: u32 = 0;
  let mut x97: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x96, &mut x97, x95, x79, x76);
  let mut x98: u32 = 0;
  let mut x99: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x98, &mut x99, x97, x77, x74);
  let mut x100: u32 = 0;
  let mut x101: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x100, &mut x101, x99, x75, x72);
  let mut x102: u32 = 0;
  let mut x103: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x102, &mut x103, 0x0, x54, x86);
  let mut x104: u32 = 0;
  let mut x105: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x104, &mut x105, x103, x56, x88);
  let mut x106: u32 = 0;
  let mut x107: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x106, &mut x107, x105, x58, x90);
  let mut x108: u32 = 0;
  let mut x109: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x108, &mut x109, x107, x60, x92);
  let mut x110: u32 = 0;
  let mut x111: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x110, &mut x111, x109, x62, x94);
  let mut x112: u32 = 0;
  let mut x113: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x112, &mut x113, x111, x64, x96);
  let mut x114: u32 = 0;
  let mut x115: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x114, &mut x115, x113, x66, x98);
  let mut x116: u32 = 0;
  let mut x117: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x116, &mut x117, x115, x68, x100);
  let mut x118: u32 = 0;
  let mut x119: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x118, &mut x119, x101, x73, (0x0 as u32));
  let mut x120: u32 = 0;
  let mut x121: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x120, &mut x121, x53, (0x0 as u32), (0x0 as u32));
  let mut x122: u32 = 0;
  let mut x123: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x122, &mut x123, x69, ((x120 as fiat_secp256k1_u1) as u32), (0x0 as u32));
  let mut x124: u32 = 0;
  let mut x125: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x124, &mut x125, x117, x122, x118);
  let mut x126: u32 = 0;
  let mut x127: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x126, &mut x127, 0x0, x104, (arg1[2]));
  let mut x128: u32 = 0;
  let mut x129: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x128, &mut x129, x127, x106, (0x0 as u32));
  let mut x130: u32 = 0;
  let mut x131: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x130, &mut x131, x129, x108, (0x0 as u32));
  let mut x132: u32 = 0;
  let mut x133: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x132, &mut x133, x131, x110, (0x0 as u32));
  let mut x134: u32 = 0;
  let mut x135: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x134, &mut x135, x133, x112, (0x0 as u32));
  let mut x136: u32 = 0;
  let mut x137: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x136, &mut x137, x135, x114, (0x0 as u32));
  let mut x138: u32 = 0;
  let mut x139: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x138, &mut x139, x137, x116, (0x0 as u32));
  let mut x140: u32 = 0;
  let mut x141: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x140, &mut x141, x139, x124, (0x0 as u32));
  let mut x142: u32 = 0;
  let mut x143: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x142, &mut x143, x126, 0xd2253531);
  let mut x144: u32 = 0;
  let mut x145: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x144, &mut x145, x142, 0xffffffff);
  let mut x146: u32 = 0;
  let mut x147: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x146, &mut x147, x142, 0xffffffff);
  let mut x148: u32 = 0;
  let mut x149: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x148, &mut x149, x142, 0xffffffff);
  let mut x150: u32 = 0;
  let mut x151: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x150, &mut x151, x142, 0xffffffff);
  let mut x152: u32 = 0;
  let mut x153: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x152, &mut x153, x142, 0xffffffff);
  let mut x154: u32 = 0;
  let mut x155: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x154, &mut x155, x142, 0xffffffff);
  let mut x156: u32 = 0;
  let mut x157: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x156, &mut x157, x142, 0xfffffffe);
  let mut x158: u32 = 0;
  let mut x159: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x158, &mut x159, x142, 0xfffffc2f);
  let mut x160: u32 = 0;
  let mut x161: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x160, &mut x161, 0x0, x159, x156);
  let mut x162: u32 = 0;
  let mut x163: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x162, &mut x163, x161, x157, x154);
  let mut x164: u32 = 0;
  let mut x165: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x164, &mut x165, x163, x155, x152);
  let mut x166: u32 = 0;
  let mut x167: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x166, &mut x167, x165, x153, x150);
  let mut x168: u32 = 0;
  let mut x169: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x168, &mut x169, x167, x151, x148);
  let mut x170: u32 = 0;
  let mut x171: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x170, &mut x171, x169, x149, x146);
  let mut x172: u32 = 0;
  let mut x173: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x172, &mut x173, x171, x147, x144);
  let mut x174: u32 = 0;
  let mut x175: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x174, &mut x175, 0x0, x126, x158);
  let mut x176: u32 = 0;
  let mut x177: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x176, &mut x177, x175, x128, x160);
  let mut x178: u32 = 0;
  let mut x179: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x178, &mut x179, x177, x130, x162);
  let mut x180: u32 = 0;
  let mut x181: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x180, &mut x181, x179, x132, x164);
  let mut x182: u32 = 0;
  let mut x183: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x182, &mut x183, x181, x134, x166);
  let mut x184: u32 = 0;
  let mut x185: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x184, &mut x185, x183, x136, x168);
  let mut x186: u32 = 0;
  let mut x187: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x186, &mut x187, x185, x138, x170);
  let mut x188: u32 = 0;
  let mut x189: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x188, &mut x189, x187, x140, x172);
  let mut x190: u32 = 0;
  let mut x191: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x190, &mut x191, x173, x145, (0x0 as u32));
  let mut x192: u32 = 0;
  let mut x193: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x192, &mut x193, x125, (0x0 as u32), (0x0 as u32));
  let mut x194: u32 = 0;
  let mut x195: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x194, &mut x195, x141, ((x192 as fiat_secp256k1_u1) as u32), (0x0 as u32));
  let mut x196: u32 = 0;
  let mut x197: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x196, &mut x197, x189, x194, x190);
  let mut x198: u32 = 0;
  let mut x199: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x198, &mut x199, 0x0, x176, (arg1[3]));
  let mut x200: u32 = 0;
  let mut x201: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x200, &mut x201, x199, x178, (0x0 as u32));
  let mut x202: u32 = 0;
  let mut x203: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x202, &mut x203, x201, x180, (0x0 as u32));
  let mut x204: u32 = 0;
  let mut x205: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x204, &mut x205, x203, x182, (0x0 as u32));
  let mut x206: u32 = 0;
  let mut x207: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x206, &mut x207, x205, x184, (0x0 as u32));
  let mut x208: u32 = 0;
  let mut x209: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x208, &mut x209, x207, x186, (0x0 as u32));
  let mut x210: u32 = 0;
  let mut x211: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x210, &mut x211, x209, x188, (0x0 as u32));
  let mut x212: u32 = 0;
  let mut x213: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x212, &mut x213, x211, x196, (0x0 as u32));
  let mut x214: u32 = 0;
  let mut x215: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x214, &mut x215, x198, 0xd2253531);
  let mut x216: u32 = 0;
  let mut x217: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x216, &mut x217, x214, 0xffffffff);
  let mut x218: u32 = 0;
  let mut x219: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x218, &mut x219, x214, 0xffffffff);
  let mut x220: u32 = 0;
  let mut x221: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x220, &mut x221, x214, 0xffffffff);
  let mut x222: u32 = 0;
  let mut x223: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x222, &mut x223, x214, 0xffffffff);
  let mut x224: u32 = 0;
  let mut x225: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x224, &mut x225, x214, 0xffffffff);
  let mut x226: u32 = 0;
  let mut x227: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x226, &mut x227, x214, 0xffffffff);
  let mut x228: u32 = 0;
  let mut x229: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x228, &mut x229, x214, 0xfffffffe);
  let mut x230: u32 = 0;
  let mut x231: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x230, &mut x231, x214, 0xfffffc2f);
  let mut x232: u32 = 0;
  let mut x233: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x232, &mut x233, 0x0, x231, x228);
  let mut x234: u32 = 0;
  let mut x235: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x234, &mut x235, x233, x229, x226);
  let mut x236: u32 = 0;
  let mut x237: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x236, &mut x237, x235, x227, x224);
  let mut x238: u32 = 0;
  let mut x239: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x238, &mut x239, x237, x225, x222);
  let mut x240: u32 = 0;
  let mut x241: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x240, &mut x241, x239, x223, x220);
  let mut x242: u32 = 0;
  let mut x243: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x242, &mut x243, x241, x221, x218);
  let mut x244: u32 = 0;
  let mut x245: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x244, &mut x245, x243, x219, x216);
  let mut x246: u32 = 0;
  let mut x247: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x246, &mut x247, 0x0, x198, x230);
  let mut x248: u32 = 0;
  let mut x249: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x248, &mut x249, x247, x200, x232);
  let mut x250: u32 = 0;
  let mut x251: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x250, &mut x251, x249, x202, x234);
  let mut x252: u32 = 0;
  let mut x253: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x252, &mut x253, x251, x204, x236);
  let mut x254: u32 = 0;
  let mut x255: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x254, &mut x255, x253, x206, x238);
  let mut x256: u32 = 0;
  let mut x257: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x256, &mut x257, x255, x208, x240);
  let mut x258: u32 = 0;
  let mut x259: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x258, &mut x259, x257, x210, x242);
  let mut x260: u32 = 0;
  let mut x261: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x260, &mut x261, x259, x212, x244);
  let mut x262: u32 = 0;
  let mut x263: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x262, &mut x263, x245, x217, (0x0 as u32));
  let mut x264: u32 = 0;
  let mut x265: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x264, &mut x265, x197, (0x0 as u32), (0x0 as u32));
  let mut x266: u32 = 0;
  let mut x267: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x266, &mut x267, x213, ((x264 as fiat_secp256k1_u1) as u32), (0x0 as u32));
  let mut x268: u32 = 0;
  let mut x269: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x268, &mut x269, x261, x266, x262);
  let mut x270: u32 = 0;
  let mut x271: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x270, &mut x271, 0x0, x248, (arg1[4]));
  let mut x272: u32 = 0;
  let mut x273: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x272, &mut x273, x271, x250, (0x0 as u32));
  let mut x274: u32 = 0;
  let mut x275: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x274, &mut x275, x273, x252, (0x0 as u32));
  let mut x276: u32 = 0;
  let mut x277: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x276, &mut x277, x275, x254, (0x0 as u32));
  let mut x278: u32 = 0;
  let mut x279: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x278, &mut x279, x277, x256, (0x0 as u32));
  let mut x280: u32 = 0;
  let mut x281: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x280, &mut x281, x279, x258, (0x0 as u32));
  let mut x282: u32 = 0;
  let mut x283: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x282, &mut x283, x281, x260, (0x0 as u32));
  let mut x284: u32 = 0;
  let mut x285: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x284, &mut x285, x283, x268, (0x0 as u32));
  let mut x286: u32 = 0;
  let mut x287: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x286, &mut x287, x270, 0xd2253531);
  let mut x288: u32 = 0;
  let mut x289: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x288, &mut x289, x286, 0xffffffff);
  let mut x290: u32 = 0;
  let mut x291: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x290, &mut x291, x286, 0xffffffff);
  let mut x292: u32 = 0;
  let mut x293: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x292, &mut x293, x286, 0xffffffff);
  let mut x294: u32 = 0;
  let mut x295: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x294, &mut x295, x286, 0xffffffff);
  let mut x296: u32 = 0;
  let mut x297: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x296, &mut x297, x286, 0xffffffff);
  let mut x298: u32 = 0;
  let mut x299: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x298, &mut x299, x286, 0xffffffff);
  let mut x300: u32 = 0;
  let mut x301: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x300, &mut x301, x286, 0xfffffffe);
  let mut x302: u32 = 0;
  let mut x303: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x302, &mut x303, x286, 0xfffffc2f);
  let mut x304: u32 = 0;
  let mut x305: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x304, &mut x305, 0x0, x303, x300);
  let mut x306: u32 = 0;
  let mut x307: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x306, &mut x307, x305, x301, x298);
  let mut x308: u32 = 0;
  let mut x309: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x308, &mut x309, x307, x299, x296);
  let mut x310: u32 = 0;
  let mut x311: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x310, &mut x311, x309, x297, x294);
  let mut x312: u32 = 0;
  let mut x313: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x312, &mut x313, x311, x295, x292);
  let mut x314: u32 = 0;
  let mut x315: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x314, &mut x315, x313, x293, x290);
  let mut x316: u32 = 0;
  let mut x317: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x316, &mut x317, x315, x291, x288);
  let mut x318: u32 = 0;
  let mut x319: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x318, &mut x319, 0x0, x270, x302);
  let mut x320: u32 = 0;
  let mut x321: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x320, &mut x321, x319, x272, x304);
  let mut x322: u32 = 0;
  let mut x323: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x322, &mut x323, x321, x274, x306);
  let mut x324: u32 = 0;
  let mut x325: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x324, &mut x325, x323, x276, x308);
  let mut x326: u32 = 0;
  let mut x327: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x326, &mut x327, x325, x278, x310);
  let mut x328: u32 = 0;
  let mut x329: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x328, &mut x329, x327, x280, x312);
  let mut x330: u32 = 0;
  let mut x331: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x330, &mut x331, x329, x282, x314);
  let mut x332: u32 = 0;
  let mut x333: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x332, &mut x333, x331, x284, x316);
  let mut x334: u32 = 0;
  let mut x335: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x334, &mut x335, x317, x289, (0x0 as u32));
  let mut x336: u32 = 0;
  let mut x337: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x336, &mut x337, x269, (0x0 as u32), (0x0 as u32));
  let mut x338: u32 = 0;
  let mut x339: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x338, &mut x339, x285, ((x336 as fiat_secp256k1_u1) as u32), (0x0 as u32));
  let mut x340: u32 = 0;
  let mut x341: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x340, &mut x341, x333, x338, x334);
  let mut x342: u32 = 0;
  let mut x343: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x342, &mut x343, 0x0, x320, (arg1[5]));
  let mut x344: u32 = 0;
  let mut x345: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x344, &mut x345, x343, x322, (0x0 as u32));
  let mut x346: u32 = 0;
  let mut x347: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x346, &mut x347, x345, x324, (0x0 as u32));
  let mut x348: u32 = 0;
  let mut x349: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x348, &mut x349, x347, x326, (0x0 as u32));
  let mut x350: u32 = 0;
  let mut x351: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x350, &mut x351, x349, x328, (0x0 as u32));
  let mut x352: u32 = 0;
  let mut x353: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x352, &mut x353, x351, x330, (0x0 as u32));
  let mut x354: u32 = 0;
  let mut x355: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x354, &mut x355, x353, x332, (0x0 as u32));
  let mut x356: u32 = 0;
  let mut x357: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x356, &mut x357, x355, x340, (0x0 as u32));
  let mut x358: u32 = 0;
  let mut x359: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x358, &mut x359, x342, 0xd2253531);
  let mut x360: u32 = 0;
  let mut x361: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x360, &mut x361, x358, 0xffffffff);
  let mut x362: u32 = 0;
  let mut x363: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x362, &mut x363, x358, 0xffffffff);
  let mut x364: u32 = 0;
  let mut x365: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x364, &mut x365, x358, 0xffffffff);
  let mut x366: u32 = 0;
  let mut x367: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x366, &mut x367, x358, 0xffffffff);
  let mut x368: u32 = 0;
  let mut x369: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x368, &mut x369, x358, 0xffffffff);
  let mut x370: u32 = 0;
  let mut x371: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x370, &mut x371, x358, 0xffffffff);
  let mut x372: u32 = 0;
  let mut x373: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x372, &mut x373, x358, 0xfffffffe);
  let mut x374: u32 = 0;
  let mut x375: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x374, &mut x375, x358, 0xfffffc2f);
  let mut x376: u32 = 0;
  let mut x377: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x376, &mut x377, 0x0, x375, x372);
  let mut x378: u32 = 0;
  let mut x379: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x378, &mut x379, x377, x373, x370);
  let mut x380: u32 = 0;
  let mut x381: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x380, &mut x381, x379, x371, x368);
  let mut x382: u32 = 0;
  let mut x383: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x382, &mut x383, x381, x369, x366);
  let mut x384: u32 = 0;
  let mut x385: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x384, &mut x385, x383, x367, x364);
  let mut x386: u32 = 0;
  let mut x387: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x386, &mut x387, x385, x365, x362);
  let mut x388: u32 = 0;
  let mut x389: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x388, &mut x389, x387, x363, x360);
  let mut x390: u32 = 0;
  let mut x391: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x390, &mut x391, 0x0, x342, x374);
  let mut x392: u32 = 0;
  let mut x393: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x392, &mut x393, x391, x344, x376);
  let mut x394: u32 = 0;
  let mut x395: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x394, &mut x395, x393, x346, x378);
  let mut x396: u32 = 0;
  let mut x397: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x396, &mut x397, x395, x348, x380);
  let mut x398: u32 = 0;
  let mut x399: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x398, &mut x399, x397, x350, x382);
  let mut x400: u32 = 0;
  let mut x401: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x400, &mut x401, x399, x352, x384);
  let mut x402: u32 = 0;
  let mut x403: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x402, &mut x403, x401, x354, x386);
  let mut x404: u32 = 0;
  let mut x405: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x404, &mut x405, x403, x356, x388);
  let mut x406: u32 = 0;
  let mut x407: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x406, &mut x407, x389, x361, (0x0 as u32));
  let mut x408: u32 = 0;
  let mut x409: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x408, &mut x409, x341, (0x0 as u32), (0x0 as u32));
  let mut x410: u32 = 0;
  let mut x411: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x410, &mut x411, x357, ((x408 as fiat_secp256k1_u1) as u32), (0x0 as u32));
  let mut x412: u32 = 0;
  let mut x413: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x412, &mut x413, x405, x410, x406);
  let mut x414: u32 = 0;
  let mut x415: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x414, &mut x415, 0x0, x392, (arg1[6]));
  let mut x416: u32 = 0;
  let mut x417: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x416, &mut x417, x415, x394, (0x0 as u32));
  let mut x418: u32 = 0;
  let mut x419: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x418, &mut x419, x417, x396, (0x0 as u32));
  let mut x420: u32 = 0;
  let mut x421: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x420, &mut x421, x419, x398, (0x0 as u32));
  let mut x422: u32 = 0;
  let mut x423: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x422, &mut x423, x421, x400, (0x0 as u32));
  let mut x424: u32 = 0;
  let mut x425: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x424, &mut x425, x423, x402, (0x0 as u32));
  let mut x426: u32 = 0;
  let mut x427: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x426, &mut x427, x425, x404, (0x0 as u32));
  let mut x428: u32 = 0;
  let mut x429: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x428, &mut x429, x427, x412, (0x0 as u32));
  let mut x430: u32 = 0;
  let mut x431: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x430, &mut x431, x414, 0xd2253531);
  let mut x432: u32 = 0;
  let mut x433: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x432, &mut x433, x430, 0xffffffff);
  let mut x434: u32 = 0;
  let mut x435: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x434, &mut x435, x430, 0xffffffff);
  let mut x436: u32 = 0;
  let mut x437: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x436, &mut x437, x430, 0xffffffff);
  let mut x438: u32 = 0;
  let mut x439: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x438, &mut x439, x430, 0xffffffff);
  let mut x440: u32 = 0;
  let mut x441: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x440, &mut x441, x430, 0xffffffff);
  let mut x442: u32 = 0;
  let mut x443: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x442, &mut x443, x430, 0xffffffff);
  let mut x444: u32 = 0;
  let mut x445: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x444, &mut x445, x430, 0xfffffffe);
  let mut x446: u32 = 0;
  let mut x447: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x446, &mut x447, x430, 0xfffffc2f);
  let mut x448: u32 = 0;
  let mut x449: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x448, &mut x449, 0x0, x447, x444);
  let mut x450: u32 = 0;
  let mut x451: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x450, &mut x451, x449, x445, x442);
  let mut x452: u32 = 0;
  let mut x453: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x452, &mut x453, x451, x443, x440);
  let mut x454: u32 = 0;
  let mut x455: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x454, &mut x455, x453, x441, x438);
  let mut x456: u32 = 0;
  let mut x457: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x456, &mut x457, x455, x439, x436);
  let mut x458: u32 = 0;
  let mut x459: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x458, &mut x459, x457, x437, x434);
  let mut x460: u32 = 0;
  let mut x461: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x460, &mut x461, x459, x435, x432);
  let mut x462: u32 = 0;
  let mut x463: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x462, &mut x463, 0x0, x414, x446);
  let mut x464: u32 = 0;
  let mut x465: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x464, &mut x465, x463, x416, x448);
  let mut x466: u32 = 0;
  let mut x467: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x466, &mut x467, x465, x418, x450);
  let mut x468: u32 = 0;
  let mut x469: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x468, &mut x469, x467, x420, x452);
  let mut x470: u32 = 0;
  let mut x471: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x470, &mut x471, x469, x422, x454);
  let mut x472: u32 = 0;
  let mut x473: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x472, &mut x473, x471, x424, x456);
  let mut x474: u32 = 0;
  let mut x475: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x474, &mut x475, x473, x426, x458);
  let mut x476: u32 = 0;
  let mut x477: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x476, &mut x477, x475, x428, x460);
  let mut x478: u32 = 0;
  let mut x479: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x478, &mut x479, x461, x433, (0x0 as u32));
  let mut x480: u32 = 0;
  let mut x481: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x480, &mut x481, x413, (0x0 as u32), (0x0 as u32));
  let mut x482: u32 = 0;
  let mut x483: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x482, &mut x483, x429, ((x480 as fiat_secp256k1_u1) as u32), (0x0 as u32));
  let mut x484: u32 = 0;
  let mut x485: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x484, &mut x485, x477, x482, x478);
  let mut x486: u32 = 0;
  let mut x487: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x486, &mut x487, 0x0, x464, (arg1[7]));
  let mut x488: u32 = 0;
  let mut x489: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x488, &mut x489, x487, x466, (0x0 as u32));
  let mut x490: u32 = 0;
  let mut x491: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x490, &mut x491, x489, x468, (0x0 as u32));
  let mut x492: u32 = 0;
  let mut x493: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x492, &mut x493, x491, x470, (0x0 as u32));
  let mut x494: u32 = 0;
  let mut x495: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x494, &mut x495, x493, x472, (0x0 as u32));
  let mut x496: u32 = 0;
  let mut x497: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x496, &mut x497, x495, x474, (0x0 as u32));
  let mut x498: u32 = 0;
  let mut x499: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x498, &mut x499, x497, x476, (0x0 as u32));
  let mut x500: u32 = 0;
  let mut x501: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x500, &mut x501, x499, x484, (0x0 as u32));
  let mut x502: u32 = 0;
  let mut x503: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x502, &mut x503, x486, 0xd2253531);
  let mut x504: u32 = 0;
  let mut x505: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x504, &mut x505, x502, 0xffffffff);
  let mut x506: u32 = 0;
  let mut x507: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x506, &mut x507, x502, 0xffffffff);
  let mut x508: u32 = 0;
  let mut x509: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x508, &mut x509, x502, 0xffffffff);
  let mut x510: u32 = 0;
  let mut x511: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x510, &mut x511, x502, 0xffffffff);
  let mut x512: u32 = 0;
  let mut x513: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x512, &mut x513, x502, 0xffffffff);
  let mut x514: u32 = 0;
  let mut x515: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x514, &mut x515, x502, 0xffffffff);
  let mut x516: u32 = 0;
  let mut x517: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x516, &mut x517, x502, 0xfffffffe);
  let mut x518: u32 = 0;
  let mut x519: u32 = 0;
  fiat_secp256k1_mulx_u32(&mut x518, &mut x519, x502, 0xfffffc2f);
  let mut x520: u32 = 0;
  let mut x521: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x520, &mut x521, 0x0, x519, x516);
  let mut x522: u32 = 0;
  let mut x523: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x522, &mut x523, x521, x517, x514);
  let mut x524: u32 = 0;
  let mut x525: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x524, &mut x525, x523, x515, x512);
  let mut x526: u32 = 0;
  let mut x527: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x526, &mut x527, x525, x513, x510);
  let mut x528: u32 = 0;
  let mut x529: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x528, &mut x529, x527, x511, x508);
  let mut x530: u32 = 0;
  let mut x531: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x530, &mut x531, x529, x509, x506);
  let mut x532: u32 = 0;
  let mut x533: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x532, &mut x533, x531, x507, x504);
  let mut x534: u32 = 0;
  let mut x535: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x534, &mut x535, 0x0, x486, x518);
  let mut x536: u32 = 0;
  let mut x537: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x536, &mut x537, x535, x488, x520);
  let mut x538: u32 = 0;
  let mut x539: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x538, &mut x539, x537, x490, x522);
  let mut x540: u32 = 0;
  let mut x541: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x540, &mut x541, x539, x492, x524);
  let mut x542: u32 = 0;
  let mut x543: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x542, &mut x543, x541, x494, x526);
  let mut x544: u32 = 0;
  let mut x545: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x544, &mut x545, x543, x496, x528);
  let mut x546: u32 = 0;
  let mut x547: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x546, &mut x547, x545, x498, x530);
  let mut x548: u32 = 0;
  let mut x549: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x548, &mut x549, x547, x500, x532);
  let mut x550: u32 = 0;
  let mut x551: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x550, &mut x551, x533, x505, (0x0 as u32));
  let mut x552: u32 = 0;
  let mut x553: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x552, &mut x553, x485, (0x0 as u32), (0x0 as u32));
  let mut x554: u32 = 0;
  let mut x555: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x554, &mut x555, x501, ((x552 as fiat_secp256k1_u1) as u32), (0x0 as u32));
  let mut x556: u32 = 0;
  let mut x557: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x556, &mut x557, x549, x554, x550);
  let mut x558: u32 = 0;
  let mut x559: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x558, &mut x559, 0x0, x536, 0xfffffc2f);
  let mut x560: u32 = 0;
  let mut x561: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x560, &mut x561, x559, x538, 0xfffffffe);
  let mut x562: u32 = 0;
  let mut x563: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x562, &mut x563, x561, x540, 0xffffffff);
  let mut x564: u32 = 0;
  let mut x565: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x564, &mut x565, x563, x542, 0xffffffff);
  let mut x566: u32 = 0;
  let mut x567: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x566, &mut x567, x565, x544, 0xffffffff);
  let mut x568: u32 = 0;
  let mut x569: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x568, &mut x569, x567, x546, 0xffffffff);
  let mut x570: u32 = 0;
  let mut x571: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x570, &mut x571, x569, x548, 0xffffffff);
  let mut x572: u32 = 0;
  let mut x573: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x572, &mut x573, x571, x556, 0xffffffff);
  let mut x574: u32 = 0;
  let mut x575: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_addcarryx_u32(&mut x574, &mut x575, x557, (0x0 as u32), (0x0 as u32));
  let mut x576: u32 = 0;
  let mut x577: fiat_secp256k1_u1 = 0;
  fiat_secp256k1_subborrowx_u32(&mut x576, &mut x577, x573, ((x574 as fiat_secp256k1_u1) as u32), (0x0 as u32));
  let mut x578: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x578, x577, x558, x536);
  let mut x579: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x579, x577, x560, x538);
  let mut x580: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x580, x577, x562, x540);
  let mut x581: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x581, x577, x564, x542);
  let mut x582: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x582, x577, x566, x544);
  let mut x583: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x583, x577, x568, x546);
  let mut x584: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x584, x577, x570, x548);
  let mut x585: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x585, x577, x572, x556);
  out1[0] = x578;
  out1[1] = x579;
  out1[2] = x580;
  out1[3] = x581;
  out1[4] = x582;
  out1[5] = x583;
  out1[6] = x584;
  out1[7] = x585;
}

/*
 * The function fiat_secp256k1_nonzero outputs a single non-zero word if the input is non-zero and zero otherwise.
 * Preconditions:
 *   0 ≤ eval arg1 < m
 * Postconditions:
 *   out1 = 0 ↔ eval (from_montgomery arg1) mod m = 0
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffff]
 */
#[inline]
pub fn fiat_secp256k1_nonzero(out1: &mut u32, arg1: &[u32; 8]) -> () {
  let x1: u32 = ((arg1[0]) | ((arg1[1]) | ((arg1[2]) | ((arg1[3]) | ((arg1[4]) | ((arg1[5]) | ((arg1[6]) | ((arg1[7]) | (0x0 as u32)))))))));
  *out1 = x1;
}

/*
 * The function fiat_secp256k1_selectznz is a multi-limb conditional select.
 * Postconditions:
 *   eval out1 = (if arg1 = 0 then eval arg2 else eval arg3)
 *
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 *   arg3: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
#[inline]
pub fn fiat_secp256k1_selectznz(out1: &mut [u32; 8], arg1: fiat_secp256k1_u1, arg2: &[u32; 8], arg3: &[u32; 8]) -> () {
  let mut x1: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x1, arg1, (arg2[0]), (arg3[0]));
  let mut x2: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x2, arg1, (arg2[1]), (arg3[1]));
  let mut x3: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x3, arg1, (arg2[2]), (arg3[2]));
  let mut x4: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x4, arg1, (arg2[3]), (arg3[3]));
  let mut x5: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x5, arg1, (arg2[4]), (arg3[4]));
  let mut x6: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x6, arg1, (arg2[5]), (arg3[5]));
  let mut x7: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x7, arg1, (arg2[6]), (arg3[6]));
  let mut x8: u32 = 0;
  fiat_secp256k1_cmovznz_u32(&mut x8, arg1, (arg2[7]), (arg3[7]));
  out1[0] = x1;
  out1[1] = x2;
  out1[2] = x3;
  out1[3] = x4;
  out1[4] = x5;
  out1[5] = x6;
  out1[6] = x7;
  out1[7] = x8;
}

/*
 * The function fiat_secp256k1_to_bytes serializes a field element in the Montgomery domain to bytes in little-endian order.
 * Preconditions:
 *   0 ≤ eval arg1 < m
 * Postconditions:
 *   out1 = map (λ x, ⌊((eval arg1 mod m) mod 2^(8 * (x + 1))) / 2^(8 * x)⌋) [0..31]
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff]]
 */
#[inline]
pub fn fiat_secp256k1_to_bytes(out1: &mut [u8; 32], arg1: &[u32; 8]) -> () {
  let x1: u32 = (arg1[7]);
  let x2: u32 = (arg1[6]);
  let x3: u32 = (arg1[5]);
  let x4: u32 = (arg1[4]);
  let x5: u32 = (arg1[3]);
  let x6: u32 = (arg1[2]);
  let x7: u32 = (arg1[1]);
  let x8: u32 = (arg1[0]);
  let x9: u32 = (x8 >> 8);
  let x10: u8 = ((x8 & (0xff as u32)) as u8);
  let x11: u32 = (x9 >> 8);
  let x12: u8 = ((x9 & (0xff as u32)) as u8);
  let x13: u8 = ((x11 >> 8) as u8);
  let x14: u8 = ((x11 & (0xff as u32)) as u8);
  let x15: u8 = (x13 & 0xff);
  let x16: u32 = (x7 >> 8);
  let x17: u8 = ((x7 & (0xff as u32)) as u8);
  let x18: u32 = (x16 >> 8);
  let x19: u8 = ((x16 & (0xff as u32)) as u8);
  let x20: u8 = ((x18 >> 8) as u8);
  let x21: u8 = ((x18 & (0xff as u32)) as u8);
  let x22: u8 = (x20 & 0xff);
  let x23: u32 = (x6 >> 8);
  let x24: u8 = ((x6 & (0xff as u32)) as u8);
  let x25: u32 = (x23 >> 8);
  let x26: u8 = ((x23 & (0xff as u32)) as u8);
  let x27: u8 = ((x25 >> 8) as u8);
  let x28: u8 = ((x25 & (0xff as u32)) as u8);
  let x29: u8 = (x27 & 0xff);
  let x30: u32 = (x5 >> 8);
  let x31: u8 = ((x5 & (0xff as u32)) as u8);
  let x32: u32 = (x30 >> 8);
  let x33: u8 = ((x30 & (0xff as u32)) as u8);
  let x34: u8 = ((x32 >> 8) as u8);
  let x35: u8 = ((x32 & (0xff as u32)) as u8);
  let x36: u8 = (x34 & 0xff);
  let x37: u32 = (x4 >> 8);
  let x38: u8 = ((x4 & (0xff as u32)) as u8);
  let x39: u32 = (x37 >> 8);
  let x40: u8 = ((x37 & (0xff as u32)) as u8);
  let x41: u8 = ((x39 >> 8) as u8);
  let x42: u8 = ((x39 & (0xff as u32)) as u8);
  let x43: u8 = (x41 & 0xff);
  let x44: u32 = (x3 >> 8);
  let x45: u8 = ((x3 & (0xff as u32)) as u8);
  let x46: u32 = (x44 >> 8);
  let x47: u8 = ((x44 & (0xff as u32)) as u8);
  let x48: u8 = ((x46 >> 8) as u8);
  let x49: u8 = ((x46 & (0xff as u32)) as u8);
  let x50: u8 = (x48 & 0xff);
  let x51: u32 = (x2 >> 8);
  let x52: u8 = ((x2 & (0xff as u32)) as u8);
  let x53: u32 = (x51 >> 8);
  let x54: u8 = ((x51 & (0xff as u32)) as u8);
  let x55: u8 = ((x53 >> 8) as u8);
  let x56: u8 = ((x53 & (0xff as u32)) as u8);
  let x57: u8 = (x55 & 0xff);
  let x58: u32 = (x1 >> 8);
  let x59: u8 = ((x1 & (0xff as u32)) as u8);
  let x60: u32 = (x58 >> 8);
  let x61: u8 = ((x58 & (0xff as u32)) as u8);
  let x62: u8 = ((x60 >> 8) as u8);
  let x63: u8 = ((x60 & (0xff as u32)) as u8);
  out1[0] = x10;
  out1[1] = x12;
  out1[2] = x14;
  out1[3] = x15;
  out1[4] = x17;
  out1[5] = x19;
  out1[6] = x21;
  out1[7] = x22;
  out1[8] = x24;
  out1[9] = x26;
  out1[10] = x28;
  out1[11] = x29;
  out1[12] = x31;
  out1[13] = x33;
  out1[14] = x35;
  out1[15] = x36;
  out1[16] = x38;
  out1[17] = x40;
  out1[18] = x42;
  out1[19] = x43;
  out1[20] = x45;
  out1[21] = x47;
  out1[22] = x49;
  out1[23] = x50;
  out1[24] = x52;
  out1[25] = x54;
  out1[26] = x56;
  out1[27] = x57;
  out1[28] = x59;
  out1[29] = x61;
  out1[30] = x63;
  out1[31] = x62;
}

/*
 * The function fiat_secp256k1_from_bytes deserializes a field element in the Montgomery domain from bytes in little-endian order.
 * Preconditions:
 *   0 ≤ bytes_eval arg1 < m
 * Postconditions:
 *   eval out1 mod m = bytes_eval arg1 mod m
 *   0 ≤ eval out1 < m
 *
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff], [0x0 ~> 0xffffffff]]
 */
#[inline]
pub fn fiat_secp256k1_from_bytes(out1: &mut [u32; 8], arg1: &[u8; 32]) -> () {
  let x1: u32 = (((arg1[31]) as u32) << 24);
  let x2: u32 = (((arg1[30]) as u32) << 16);
  let x3: u32 = (((arg1[29]) as u32) << 8);
  let x4: u8 = (arg1[28]);
  let x5: u32 = (((arg1[27]) as u32) << 24);
  let x6: u32 = (((arg1[26]) as u32) << 16);
  let x7: u32 = (((arg1[25]) as u32) << 8);
  let x8: u8 = (arg1[24]);
  let x9: u32 = (((arg1[23]) as u32) << 24);
  let x10: u32 = (((arg1[22]) as u32) << 16);
  let x11: u32 = (((arg1[21]) as u32) << 8);
  let x12: u8 = (arg1[20]);
  let x13: u32 = (((arg1[19]) as u32) << 24);
  let x14: u32 = (((arg1[18]) as u32) << 16);
  let x15: u32 = (((arg1[17]) as u32) << 8);
  let x16: u8 = (arg1[16]);
  let x17: u32 = (((arg1[15]) as u32) << 24);
  let x18: u32 = (((arg1[14]) as u32) << 16);
  let x19: u32 = (((arg1[13]) as u32) << 8);
  let x20: u8 = (arg1[12]);
  let x21: u32 = (((arg1[11]) as u32) << 24);
  let x22: u32 = (((arg1[10]) as u32) << 16);
  let x23: u32 = (((arg1[9]) as u32) << 8);
  let x24: u8 = (arg1[8]);
  let x25: u32 = (((arg1[7]) as u32) << 24);
  let x26: u32 = (((arg1[6]) as u32) << 16);
  let x27: u32 = (((arg1[5]) as u32) << 8);
  let x28: u8 = (arg1[4]);
  let x29: u32 = (((arg1[3]) as u32) << 24);
  let x30: u32 = (((arg1[2]) as u32) << 16);
  let x31: u32 = (((arg1[1]) as u32) << 8);
  let x32: u8 = (arg1[0]);
  let x33: u32 = ((x32 as u32) + (x31 + (x30 + x29)));
  let x34: u32 = (x33 & 0xffffffff);
  let x35: u32 = ((x4 as u32) + (x3 + (x2 + x1)));
  let x36: u32 = ((x8 as u32) + (x7 + (x6 + x5)));
  let x37: u32 = ((x12 as u32) + (x11 + (x10 + x9)));
  let x38: u32 = ((x16 as u32) + (x15 + (x14 + x13)));
  let x39: u32 = ((x20 as u32) + (x19 + (x18 + x17)));
  let x40: u32 = ((x24 as u32) + (x23 + (x22 + x21)));
  let x41: u32 = ((x28 as u32) + (x27 + (x26 + x25)));
  let x42: u32 = (x41 & 0xffffffff);
  let x43: u32 = (x40 & 0xffffffff);
  let x44: u32 = (x39 & 0xffffffff);
  let x45: u32 = (x38 & 0xffffffff);
  let x46: u32 = (x37 & 0xffffffff);
  let x47: u32 = (x36 & 0xffffffff);
  out1[0] = x34;
  out1[1] = x42;
  out1[2] = x43;
  out1[3] = x44;
  out1[4] = x45;
  out1[5] = x46;
  out1[6] = x47;
  out1[7] = x35;
}

